/*******************************************************************************************
//
// File				Event.cpp
//
// Author:			Lukasz Beca
// Created:			--/--/--
//
// Modified:		04/13/98
// By:				Grzegorz Lewandowski
// Changes:			Win32 port
//					writeAll added
//					sendEvent returns HRESULT value
//
// Modified:		04/15/98
// By:				Grzegorz Lewandowski
// Changes:			new version of registerApp
//					registerWinApp added
//
//*****************************************************************************************/

#include "Event.h"


/*
// readAll
*/
HRESULT readAll( SOCKET fd, char* pBuffer, size_t Length )
{
	int nResult;
	do
	{
		nResult = recv( fd, pBuffer, Length, 0 );
		if( nResult > 0 )
		{
			Length -= nResult;
			pBuffer = pBuffer + nResult;
		}
		else
		{
			if( nResult == SOCKET_ERROR )
			{
				return ERROR_READ_FAULT;
			}
			return ERROR_HANDLE_EOF;			
		}
	}
	while( Length > 0 );
	return S_OK;
}


/*
// writeAll
*/
HRESULT writeAll( SOCKET fd, char* pBuffer, size_t Length )
{
	int nResult;
	do
	{
		nResult = send( fd, pBuffer, Length, 0 );
		if( nResult > 0 )
		{
			Length -= nResult;
			pBuffer = pBuffer + nResult;
		}
		else
		{
			if( nResult == SOCKET_ERROR )
			{
				return ERROR_WRITE_FAULT;
			}
			return ERROR_HANDLE_EOF;
		}
	}
	while( Length > 0 );
	return S_OK;
}


/*
// registerApp
*/
SOCKET registerApp( int argc, char *argv[] )
{
	int AID;
	int SID;
	int AT;
	int port;
	
	SOCKET sockfd;
	SOCKADDR_IN serv_addr;
	char *pBuffer;
	long lToSend;
	int nResult;
	char HostName[ 127 ];
	struct hostent *pHostInfo;

	/* retrieve command line parameters */
	if( argc < 5 )
	{
		return INVALID_SOCKET;
	}
	
	if( argv[ 1 ] != NULL )
	{
		AID = atoi( argv[ 1 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}
	
	if( argv[ 2 ] != NULL )
	{
		SID = atoi( argv[ 2 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}

	if( argv[ 3 ] != NULL )
	{
		AT = atoi( argv[ 3 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}

	if( argv[ 4 ] != NULL )
	{
		port = atoi( argv[ 4 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}

	/* obtain address of the local host */
	if( gethostname( HostName, 127 ) == SOCKET_ERROR )
	{
		return INVALID_SOCKET;
	}
	
	if( ( pHostInfo = gethostbyname( HostName ) ) == NULL )
	{
		return INVALID_SOCKET;
	}

	/* create socket */
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr = *(struct in_addr*)( pHostInfo->h_addr ); 
	serv_addr.sin_port = htons( (unsigned short)port );

	if( ( sockfd = socket( AF_INET, SOCK_STREAM, 0 ) ) == INVALID_SOCKET )
	{
		return INVALID_SOCKET;
	}

	/* connect to TANGO */
	if( connect( sockfd, (PSOCKADDR)&serv_addr, sizeof( serv_addr ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/* send identification */
	/*   AID */
	lToSend = AID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  SID */
	lToSend = SID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  AT */
	lToSend = AT;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}
	
	return sockfd;
}


/*
// registerWinApp
*/
SOCKET registerWinApp( LPSTR lpCmdLine )
{
	char szTokens[] = " ";
	char* szToken;

	int AID = -1;
	int SID = -1;
	int AT = -1;
	int port = -1;

	SOCKET sockfd;
	SOCKADDR_IN serv_addr;
	char *pBuffer;
	long lToSend;
	int nResult;
	char HostName[ 127 ];
	struct hostent *pHostInfo;

	/* retrieve command line parameters */
	szToken = strtok( lpCmdLine, szTokens ); 
	while( szToken != NULL )
	{
		if( AID == -1 )
		{
			AID = atoi( szToken );
		}
		else if( SID == -1 )
		{
			SID = atoi( szToken );
		}
		else if( AT == -1 )
		{
			AT = atoi( szToken );
		}
		else if( port == -1 )
		{
			port = atoi( szToken );
		}
		else
		{
			break;
		}
		szToken = strtok( NULL, szTokens ); 
	}

	if( ( AID == -1 ) || ( SID == -1 ) || ( AT == -1 ) || ( port == -1 ) )
	{
		return INVALID_SOCKET;
	}

	/* obtain address of the local host */
	if( gethostname( HostName, 127 ) == SOCKET_ERROR )
	{
		return INVALID_SOCKET;
	}
	
	if( ( pHostInfo = gethostbyname( HostName ) ) == NULL )
	{
		return INVALID_SOCKET;
	}

	/* create socket */
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr = *(struct in_addr*)( pHostInfo->h_addr ); 
	serv_addr.sin_port = htons( (unsigned short)port );

	if( ( sockfd = socket( AF_INET, SOCK_STREAM, 0 ) ) == INVALID_SOCKET )
	{
		return INVALID_SOCKET;
	}

	/* connect to TANGO */
	if( connect( sockfd, (PSOCKADDR)&serv_addr, sizeof( serv_addr ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/* send identification */
	/*   AID */
	lToSend = AID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  SID */
	lToSend = SID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  AT */
	lToSend = AT;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}
	
	return sockfd;
}


/*
// sendEvent
*/
HRESULT sendEvent( SOCKET sockfd, struct Event *pev )
{
	int buffer;
	HRESULT hr;

	buffer = htonl( pev->type );

	hr = writeAll( sockfd, (char*)&buffer, sizeof( int ) );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	buffer = htonl( pev->len );
	hr = writeAll( sockfd, (char*)&buffer, sizeof( int ) );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	hr = writeAll( sockfd, pev->data, pev->len );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	return S_OK;
}


/*
// receiveEvent
*/
struct Event* receiveEvent( SOCKET sockfd )
{
	HRESULT hr;
	char Buffer[ 4 ];
	int *a;
	int type;
	int len;
	char *data;
	struct Event *r; 
	
	hr = readAll( sockfd, Buffer, 4 );
	if( hr != S_OK )
	{
		return NULL;
	}
	a = (int*)Buffer;
	type = ntohl( *a );
	
	hr = readAll( sockfd, Buffer, 4 );
	if( hr != S_OK )
	{
		return NULL;
	}
	a = (int*)Buffer;
	len = ntohl( *a );
	
	data = malloc( len * sizeof( char ) );
	
	hr = readAll( sockfd, data, len );
	if( hr != S_OK )
	{
		free( data );
		return NULL;
	}
	r = createEvent( type, len, data );
	if( r == NULL )
	{
		free( data );
		return NULL;
	}
	
	if( type == PING )
	{
		if( sendEvent( sockfd, r ) != S_OK )
		{
			free( data );
			return NULL;
		}
	}

	free( data );
	return r;
}


/*
// createEvent
*/
struct Event* createEvent( int type, int len, char *data )
{
	int i;
	struct Event *Result;
	
	Result = NULL;
	Result = malloc( sizeof( struct Event ) );
	if( Result == NULL )
	{
		return NULL;
	}
	Result->type = type;
	Result->len = len;
	Result->data = NULL;
	Result->data = malloc( len * sizeof( char ) );
	if( Result->data == NULL )
	{
		return NULL;
	}

	for( i = 0; i < len; i ++ )
	{
		Result->data[ i ] = data[ i ];
	}
	
	return Result;
}


/*
// destroyEvent
*/
void destroyEvent( struct Event *pev )
{
	free( pev->data );
	free( pev );
	pev = NULL;
}


/*
// getType
*/
int getType( struct Event *pev )
{
	return pev->type;
}


/*
// getLen
*/
int getLen( struct Event *pev )
{
	return pev->len;
}


/*
// getData
*/
char *getData( struct Event *pev )
{
	int nLength;
	int i;
	char *pBuffer;

	nLength = getLen( pev );
	pBuffer = malloc( nLength * sizeof( char ) );

	for( i = 0; i < nLength; i ++ ) 
	{
		pBuffer[ i ] = pev->data[ i ];
	}
	
	return pBuffer;
}
