

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h> 
#include <netinet/in.h>
#include <arpa/inet.h>

#include "Event.h"


void readAll(register int fd,register void* buffer,register size_t len)
{
  register int st;
  do
  {
    st=read(fd,buffer,len);
    if(st>0)
    {
      len-=st;
      buffer=(char*)buffer+st;
    } 
    else
    {
      if(st<0)
	perror("readAll");
      exit(1);
    }
  }
  while(len);
}/* readAll ***************************************************************/


int registerApp(int argc,char *argv[])
{
  int sockfd;
  struct sockaddr_in serv_addr;
  char *tmp;
  int q;
  int AID;
  int SID;
  int AT;
  int port;

  if (argc != 5)
  {
    perror("invalid arguments");
    exit(0);
  }
  if(argv[1] != 0)
  {
    AID = atoi(argv[1]);
  }
  else
  {
    perror("null AID argument");
    exit(0);
  }
  if(argv[2] != 0)
  {
    SID = atoi(argv[2]);
  }
  else
  {
    perror("null SID argument");
    exit(0);
  }
  if(argv[3] != 0)
  {
    AT = atoi(argv[3]);  
  }
  else
  {
    perror("null AT argument");
    exit(0);
  }
  if(argv[4] != 0)
  {
    port = atoi(argv[4]);  
  }
  else
  {
    perror("null port argument");
    exit(0);
  }
  bzero ((char *) &serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = PF_INET;
  serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  serv_addr.sin_port = htons(port);
  if ( (sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0)
  {
    perror("socket");
    exit(0);
  }
  if(connect(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0)
  {
    perror("connect");
    exit(0);
  }
  tmp = (char *) &AID;
  q = write(sockfd, tmp, 4);     
  if (q == -1)
  {
    perror("write"); 
    exit(0);
  }
  tmp = (char *) &SID;
  q = write(sockfd, tmp, 4);     
  if (q == -1)
  {
    perror("write"); 
    exit(0);
  }
  tmp = (char *) &AT;
  q = write(sockfd, tmp, 4);     
  if (q == -1)
  {
    perror("write"); 
    exit(0);
  }
  return sockfd;
}/* registerApp ***********************************************************/


void sendEvent(int sockfd,struct Event *e)
{
  char *tmp;
  int d;
  int q;

  d=e->type;
  tmp=(char *)&d;
  q=write(sockfd,tmp,4);
  if(q==-1)
  {
    perror("write");
    exit(0);
  }
  
  d=e->len;
  tmp=(char *)&d;
  q=write(sockfd,tmp,4);
  if(q==-1)
  {
    perror("write");
    exit(0);
  }
  
  q=write(sockfd,e->data,getLen(e));
  if(q==-1)
  {
    perror("write");
    exit(0);
  }
}/* sendEvent *************************************************************/


struct Event *receiveEvent(int sockfd)
{
  struct Event *r;
  int type;


  char tmp[4];
  int *a;
  int len;
  char *data;
  
  readAll(sockfd, tmp, 4);
  a = (int *) tmp;
  type = *a;
  
  readAll(sockfd, tmp, 4);
  a = (int *)tmp;
  len = *a;
  data = (char *) malloc(len * sizeof(char));
  readAll(sockfd, data, len);
  r=createEvent(type, len, data);
  if(type==PING)
    sendEvent(sockfd,r);
  free(data);


  return r;
}/* receiveEvent **********************************************************/


struct Event *createEvent(int type,int len,char *data)
{
  int i;
  struct Event *result;

  result = (struct Event *) malloc(sizeof(struct Event));
  
  result->type = type;
  result->len = len;

  result->data = (char *) malloc(len * sizeof(char));

  for (i = 0; i < len; i ++) 
    result->data[i] = data[i];

  return result;
}/* createEvent **********************************************************/


void destroyEvent(struct Event *e)
{
  free(e->data);
  free(e);
  e = NULL;
}/* destroyEvent *********************************************************/


int getType(struct Event *e)
{
  return e->type;
}/* getType **************************************************************/


int getLen(struct Event *e)
{
  return e->len;
}/* getLen ***************************************************************/


char *getData(struct Event *e)
{
  int lentmp;
  int i;
  char *R;

  lentmp = getLen(e);
  R = (char *) malloc(lentmp * sizeof(char));
  for(i = 0; i < lentmp; i ++) 
    R[i] = e->data[i];
  return R;
}/* getData **************************************************************/


