/***************************************************************************
  $Id: TAgent.java,v 1.18 1998/10/13 18:18:30 toma Exp $
***************************************************************************/


package webwisdom.tango;


import java.applet.Applet;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.CAppAnswerEventMessage;
import webwisdom.tango.messages.CAppRequestEventMessage;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.AppletHandle;

import netscape.javascript.JSObject;


class TControlSheduler implements Runnable
{
    private static final String CL="TControlSheduler";


    Vector queue;
    TAgent agent;
    boolean isRunning;


    TControlSheduler(TAgent ta)
    {
      isRunning=false;
      queue=new Vector();
      this.agent=ta;
    }// TControlSheduler __________________________________________________


    void add(CAppAnswerEventMessage m)
    {
      synchronized(queue)
      {
	queue.addElement(m);
	queue.notify();
      }
    }// add _______________________________________________________________


    public void run()
    {
      isRunning=true;
      while(isRunning)
      {
	CAppAnswerEventMessage m=null;
	try
	{
	  synchronized(queue)
	  {
	    while(queue.size()==0 && isRunning==true)
	      queue.wait();
	    if(!isRunning)
	      return;
	    m=(CAppAnswerEventMessage)queue.elementAt(0);
	    queue.removeElementAt(0);
	  }
	  agent.handleControl(m);
	}
	catch(InterruptedException e)
	{
	  System.err.println(CL+": exception in processing control message!: "+m);
	  e.printStackTrace();
	}
      }
    }// run _______________________________________________________________


    void stop()
    {
      isRunning=false;
      synchronized(queue)
      {
	queue.notify();
      }
    }// stop ______________________________________________________________


}// TControlSheduler //////////////////////////////////////////////////////


class Semo
{
    private static final String CL="Semo";


    CAppRequestEventMessage msg;
    TAgent tn;
    byte code;
    boolean waiting;
    boolean response_available;
    CAppAnswerEventMessage res;


    public Semo(TAgent tn,byte code)
    {
      this.code=code;
      this.tn=tn;
      byte b[]=new byte[1];
      b[0]=code;
      msg=new CAppRequestEventMessage(b);
      waiting=false;
      response_available=false;
    }// Semo ______________________________________________________________
  

    public synchronized void response(CAppAnswerEventMessage res)
    {
      this.res=res;
      response_available=true;
      notifyAll();
    }// response __________________________________________________________


    public CAppAnswerEventMessage request()
    {
      synchronized(this)
      {
	waiting=true;
	response_available=false;
      }
      tn.send(msg);
      synchronized(this)
      {
	try
	{
	  while(response_available==false || res==null)
	    wait();
	}
	catch(Exception e)
	{
	}
	waiting=false;
      }
      return res;
    }// request ___________________________________________________________
    

//     public synchronized boolean check()
//     {
//       return waiting;
//     }// check _____________________________________________________________


}// Semo //////////////////////////////////////////////////////////////////


/**General Tango system proxy.
   This interface is the same for all application types.*/
public class TAgent implements TLListener
{
    private static final String CL="TAgent";


    //from ca: LocalConsts.PARTICIPANTS_UIDS
    private static final byte PARTIPUID=125;
    private static final byte AUDIO_OFF=121;
    private static final byte AUDIO_ON=123;


    protected Vector dataList;
    //protected Vector crosList;
    protected Vector contList;

    protected String userName;
    protected String masterName;
    protected Boolean isMaster;
    protected String hostName;
    private boolean isAudioAvailable=true;
    private Hashtable partipuids=new Hashtable();

    private Semo s_isMaster;
    private Semo s_masterName;
    private Semo s_userName;
    private Semo s_hostName;
    private Semo s_participants;
    private Semo s_partipuid;
    private Semo s_activeUsers;
    private Semo s_localApps;

    private TControlSheduler tScheduler;
    private TLAgent lAgent;

    private boolean v_isMaster;
    private Vector v_participants;


    /**
     *@param a low level Tango system proxy
     specific for the application type*/
    public TAgent(TLAgent a)
    {
      lAgent=a;
      dataList=new Vector();
      //crosList=new Vector();
      contList=new Vector();
      tScheduler=new TControlSheduler(this);
      s_isMaster=new Semo(this,CAConsts.ISMASTER);
      s_masterName=new Semo(this,CAConsts.MASTERNAME);
      s_userName=new Semo(this,CAConsts.USERNAME);
      s_hostName=new Semo(this,CAConsts.HOSTNAME);
      s_participants=new Semo(this,CAConsts.PARTICIPANTS);
      s_partipuid=new Semo(this,PARTIPUID);
      s_activeUsers=new Semo(this,CAConsts.ACTIVEUSERS);
      s_localApps=new Semo(this,CAConsts.LOCALAPPS);
      (new Thread(tScheduler)).start();
      lAgent.add(this);
      v_participants=getParticipants();
      v_isMaster=isMaster();
    }// TAgent ____________________________________________________________


    /**Leaves Tango system. Cannot join Tango again.*/
    public void exit()
    {
      lAgent.exit();
    }// exit ______________________________________________________________


    /**Adds a recipient of the session data.
     *@see #send()*/
    public void addTDataListener(TDataListener l)
    {
      dataList.removeElement(l);
      dataList.addElement(l);
    }// addTDataListener __________________________________________________


    public void removeTDataListener(TDataListener l)
    {
      dataList.removeElement(l);
    }// removeTDataListener _______________________________________________


//     public void addTCrossListener(TCrossListener l)
//     {
//       crosList.removeElement(l);
//       crosList.addElement(l);
//     }// addTCrossListener _________________________________________________


//     public void removeTCrossListener(TCrossListener l)
//     {
//       crosList.removeElement(l);
//     }// removeTCrossListener ______________________________________________


    /**Adds a recipient of the Tango system notifications.*/
    public void addTControlListener(TControlListener l)
    {
      contList.removeElement(l);
      contList.addElement(l);
    }// addTControlListener _______________________________________________


    public void removeTControlListener(TControlListener l)
    {
      contList.removeElement(l);
    }// removeTDataListener _______________________________________________


    /**Interface for low level mechanisms.
     *@see TLAgent*/
    public final void receive(AppEventMessage m)
    {
      //System.out.println(CL+".receive(): "+m+","+m.getType());
      if(m.getType()==Const.DATA)
      {
	for(int i=0;i<dataList.size();i++)
	  ((TDataListener)dataList.elementAt(i)).receive(m.getData());
      }
      else if(m.getType()==Const.CONTROL)
      {
	CAppAnswerEventMessage mm=(CAppAnswerEventMessage)m;
// 	if(mm.getAnswerType()==CAConsts.DATAPASS)
// 	{
// 	  String sender=mm.getSender();
// 	  byte[] data=mm.getData();
// 	  for(int i=0;i<crosList.size();i++)
// 	    ((TCrossListener)crosList.elementAt(i)).crossReceive(sender,data);
// 	}
// 	else
	{
	  handleSyncControl(mm);
	  tScheduler.add(mm);
	}
      }
    }// receive ___________________________________________________________


    private final void handleSyncControl(CAppAnswerEventMessage m)
    {
      switch(m.getAnswerType())
      {
	case CAConsts.ISMASTER:
	  masterName=null;
	  isMaster=new Boolean(m.isMaster());
	  s_isMaster.response(m);
	  break;
	case CAConsts.PARTICIPANTS:
	  s_participants.response(m);
	  break;
	case PARTIPUID:
	  byte b[]=m.getData();
	  ByteArrayInputStream bais=new ByteArrayInputStream(b);
	  DataInputStream dis=new DataInputStream(bais);
	  partipuids.clear();
	  try
	  {
	    while(dis.available()>0)
	    {
	      Integer i=new Integer(dis.readInt());
	      String s=dis.readUTF();
	      partipuids.put(s,i);
	    }
	  }
	  catch(IOException e)
	  {
	    System.err.println(CL+": error processing PARTIPUID message!: "+e);
	    e.printStackTrace();
	  }
	  //System.out.println("Got:"+ht);
	  s_partipuid.response(m);
	  break;
	case CAConsts.MASTERNAME:
	  masterName=m.getMastername();
	  s_masterName.response(m);
	  break;
	case CAConsts.USERNAME:
	  userName=m.getUsername();
	  s_userName.response(m);
	  break;
// 	case CAConsts.HOSTNAME:
// 	  hostName=m.getHostname();
// 	  s_hostName.response(m);
// 	  break;
// 	case CAConsts.ACTIVEUSERS:
// 	  s_activeUsers.response(m) ;
// 	  for(int i=0;i<contList.size();i++)
// 	    ((TControlListener)contList.elementAt(i)).usersChanged();
// 	  break;
// 	case CAConsts.LOCALAPPS:
// 	  s_localApps.response(m);
// 	  break;
	default:
	  System.err.println(CL+".handleSyncControl("+m.getAnswerType()+"): unknown message!");
      }
    }// handleSyncControl _________________________________________________


    final void handleControl(CAppAnswerEventMessage m)
    {
      switch(m.getAnswerType())
      {
	case CAConsts.ISMASTER:
	  if(m.isMaster()!=v_isMaster)
	  {
	    v_isMaster=!v_isMaster;
	    for(int i=0;i<contList.size();i++)
	      ((TControlListener)contList.elementAt(i)).masterChanged(isMaster(),getMasterName());
	  }
	  break;
	case CAConsts.PARTICIPANTS:
	  if(v_participants==null)
	  {
	    v_participants=m.getParticipants();
	    return;
	  }
	  Object o1=new Integer(1);//entered
	  Object o2=new Integer(2);//left
	  Object o3=new Integer(3);//unchanged
	  Vector participants=m.getParticipants();
	  Hashtable h_parts=new Hashtable();
	  for(int i=0;i<participants.size();i++)
	  {
	    h_parts.put(participants.elementAt(i),o1);
	  }
	  for(int i=0;i<v_participants.size();i++)
	  {
	    Object code=h_parts.get(v_participants.elementAt(i));
	    if(code==null)
	      h_parts.put(v_participants.elementAt(i),o2);
	    else
	      h_parts.put(v_participants.elementAt(i),o3);
	  }
	  Enumeration e=h_parts.keys();
	  while(e.hasMoreElements())
	  {
	    String key=(String)e.nextElement();
	    Object code=h_parts.get(key);
	    if(code==o1)
	    {
	      for(int i=0;i<contList.size();i++)
		((TControlListener)contList.elementAt(i)).participantJoined(key);
	    }
	    else if(code==o2)
	    {
	      for(int i=0;i<contList.size();i++)
		((TControlListener)contList.elementAt(i)).participantLeft(key);
	    }
	  }
	  h_parts.clear();
	  v_participants=participants;
	  break;
	case AUDIO_ON:
	case AUDIO_OFF:
	  isAudioAvailable=(m.getAnswerType()==AUDIO_ON);
	  for(int i=0;i<contList.size();i++)
	    ((TControlListener)contList.elementAt(i)).audioRequested(!isAudioAvailable);
	  break;
// 	case CAConsts.ACTIVEUSERS:
// 	  for(int i=0;i<contList.size();i++)
// 	    ((TControlListener)contList.elementAt(i)).usersChanged();
// 	  break;
	default:
	  System.err.println(CL+".handleControl("+m.getAnswerType()+"): unknown message!");
      }
    }// handleControl _____________________________________________________


    void send(AppEventMessage m)
    {
      lAgent.send(m);
    }// send ______________________________________________________________


    /**Sends data to the other participants of the same session.
     *@param data data to be sent
     *@see TDataListener*/
    public void send(byte data[])
    {
      send(new AppEventMessage(Const.DATA,data));
    }// send ______________________________________________________________


    /**Sends data to the specified participants.
     *@param part names of the recipients participating in the same session
     *@param data data to be sent
     *@see #send*/
    public void selectiveSend(String[] part,byte data[])
    {
      int n=part.length;
      int[] uids=new int[n];
      for(int i=0;i<n;i++)
	uids[i]=getUID(part[i]);
      lAgent.send(uids,new AppEventMessage(Const.DATA,data));
    }// selectiveSend _____________________________________________________


//     /**Sends data to the other session.
//      *@param sess the name of the recipient session
//      *@param data data to be sent
//      *@see TCrossListener*/
//     public final void crossSend(String sess,byte data[])
//     {
//       send(new CAppRequestEventMessage(CAppRequestEventMessage.prepareData(sess,data)));
//     }// crossSend _________________________________________________________


    /**Finds participant id.
     *@param n participant name
     *@return -1 if no such participant*/
    private int getUID(String n)
    {
      Object i=partipuids.get(n);
      if(i==null)
      {
	s_partipuid.request();
	Object ii=partipuids.get(n);
	if(ii==null)
	  return -1;
	else
	  return ((Integer)ii).intValue();
      }
      else
	return ((Integer)i).intValue();
    }// getUID ____________________________________________________________


    /**Retrieves this user name.
     *@return user name*/
    public String getUserName()
    {
      //System.out.println(">getUserName()");
      if(userName==null)
	s_userName.request();
      //System.out.println("<getUserName()");
      return userName;
    }// getUserName _______________________________________________________


//     public String[] getUserNames()
//     {
//       return toStringArray(s_activeUsers.request().getActiveUsers());
//     }// getUserNames ______________________________________________________


//     public String[] getSessionNames()
//     {
//       return toStringArray(s_localApps.request().getLocalApps());
//     }// getSessionNames ___________________________________________________
  

    Vector getParticipants()
    {
      return s_participants.request().getParticipants();
    }// getParticipants ___________________________________________________


    /**Retrieves all participants of this session.
     *@return participant names*/
    public String[] getParticipantNames()
    {
      return toStringArray(getParticipants());
    }// getParticipantNames _______________________________________________


    /**Retrieves the master of this session.
     *@return the master name*/
    public String getMasterName()
    {
      //System.out.println(">getMasterName()");
      if(masterName==null)
	s_masterName.request();
      //System.out.println("<getMasterName()");
      return masterName;
    }// getMasterName _____________________________________________________


    /**Checks whether this participant is the master of this session.
     *@return master status*/
    public boolean isMaster()
    {
      //System.out.println(">getIsMaster()");
      if(isMaster==null)
	s_isMaster.request();
      //System.out.println("<getIsMaster()");
      return isMaster.booleanValue();
    }// isMaster __________________________________________________________


    /**Checks whether this application is allowed to use audio at this moment.
     *@return true if audio can be used*/
    public boolean isAudioAvailable()
    {
      return isAudioAvailable;
    }// isAudioAvailable __________________________________________________


    private static String[] toStringArray(Vector v)
    {
      int n=v.size();
      String[] a=new String[n];
      for(int i=0;i<n;i++)
	a[i]=v.elementAt(i).toString();
      return a;
    }// toStringArray _____________________________________________________


    public String toString()
    {
      return CL+"[userName="+getUserName()+",isMaster="+isMaster()+
	",lAgent="+lAgent+"]";
    }// toString ___________________________________________________________


}// TAgent ////////////////////////////////////////////////////////////////


