/***************************************************************************
  $Id: TAgentJS.java,v 1.2 1998/11/03 00:53:02 piotr Exp $
***************************************************************************/


package webwisdom.tango;


import java.util.Vector;

import netscape.javascript.JSObject;
//import netscape.javascript.JSException;


/**Tango system proxy to be used in JavaScript applications.
   <br>Only methods ending with "JS" are designed to be called from scripts.
   *@see TAgent*/
public class TAgentJS extends TAgent
  implements TDataListener, TControlListener
{
    private static final String CL="TAgentJS";


    protected JSObject script=null;

    protected Vector dataListScr=new Vector();
    protected Vector contListScr=new Vector();


    /**
     *@param scr script - main application window
     which will receive focus and will closed on termination*/
    private TAgentJS(TLAgent a,JSObject scr)
    {
      super(a);
      script=scr;
    }// TAgentJS ___________________________________________________________


    /**Deregisters this application.*/
    public void exitJS()
    {
      exit();
    }// exitJS _____________________________________________________________


    /**Adds a recipient of the session data.
     *@param l script/window receiving application messages.
     The given window must implement <tt>Tango_receive()</tt> method.*/
    public void addTDataListenerJS(JSObject l)
    {
      dataListScr.removeElement(l);
      dataListScr.addElement(l);
      addTDataListener(this);
    }// addTDataListenerJS _________________________________________________


    /**Stops receiving private data by the given script/window.
       This function reverses the effect of <tt>addTDataListener()</tt>
       with the same argument.
       A listener can be removed and added as often as necessary.
       *@param l listener script/window
       *@see #addTDataListenerJS*/
    public void removeTDataListenerJS(JSObject l)
    {
      dataListScr.removeElement(l);
      if(dataListScr.size()==0)
	removeTDataListener(this);
    }// removeTDataListenerJS ______________________________________________


    /**Adds a recipient of the Tango system notifications.
     *@param l listener script/window - this script has to implement
     the control listener interface:
     <ul>
     <li><tt>Tango_masterChanged(isMaster,masterName);</tt>
     <li><tt>Tango_participantJoined(participantName);</tt>
     <li><tt>Tango_participantLeft(participantName);</tt>
     <li><tt>Tango_audioRequested(isAudioRequested);</tt>
     </ul>*/
    public void addTControlListenerJS(JSObject l)
    {
      contListScr.removeElement(l);
      contListScr.addElement(l);
      addTControlListener(this);
    }// addTControlListenerJS ______________________________________________


    /**Stops receiving control data by the given script/window.
       This function reverses the effect of <tt>addTControlListener()</tt>
       with the same argument.
       A listener can be removed and added as often as necessary.
       *@param l listener script/window
       *@see #addTControlListenerJS*/
    public void removeTControlListenerJS(JSObject l)
    {
      contListScr.removeElement(l);
      if(contListScr.size()==0)
	removeTControlListener(this);
    }// removeTControlListenerJS ___________________________________________


    private void channelSend(int chId,byte[] data)
    {
      int dataLen=data.length;
      byte[] chData=new byte[1+dataLen];
      chData[0]=(byte)chId;
      System.arraycopy(data,0,chData,1,dataLen);
      super.send(chData);
    }// channelSend ________________________________________________________


    private void channelSelectiveSend(int chId,String[] part,byte[] data)
    {
      int dataLen=data.length;
      byte[] chData=new byte[1+dataLen];
      chData[0]=(byte)chId;
      System.arraycopy(data,0,chData,1,dataLen);
      super.selectiveSend(part,chData);
    }// channelSelectiveSend _______________________________________________


    private static final int DEFAULT_CHANNEL=-128;


    public void send(byte[] data)
    {
      channelSend(DEFAULT_CHANNEL,data);
    }// send _______________________________________________________________


    public final void selectiveSend(String[] part,byte[] data)
    {
      channelSelectiveSend(DEFAULT_CHANNEL,part,data);
    }// selectiveSend ______________________________________________________


    /*The following 3 functions concatenate channel id and channel data
      into one message:
      first byte - channel number, remaining bytes - data.
      As channels are represented by one byte
      the channel number must be within [-128,127].
      Api with Strings uses channels [-128,-1], with bytes - [0,127].
      Default channel for Strings api without channels is -128.
      User uses channels [0,127] only,
      the rest is used to save information which api is in use.*/


    /**Sends string messages.
     *@param data string to be sent*/
    public void sendJS(String data)
    {
      send(stringToBytes(data));
    }// sendJS _____________________________________________________________


    /**Sends string messages to selected users.
     *@param data string to be sent
     *@param part participants which are to receive the message
     *@see #getParticipantNamesJS*/
    public void selectiveSendJS(String[] part,String data)
    {
      selectiveSend(part,stringToBytes(data));
    }// selectiveSendJS ____________________________________________________


    /**Sends byte array with the identifier (through the named channel).
       Thsi function is useful in sending opaque data received from applets.
     *@param chName channel name (must represent a number 1-127) eg "13"
     *@param chData channel data*/
    public void channelSendJS(String chName,byte[] data)
    {
      int chId=Integer.valueOf(chName).intValue();
      channelSend(chId,data);
    }// channelSendJS ______________________________________________________


    public void receive(byte data[])
    {
      int chId=(int)data[0];
      int chDataLen=data.length-1;
      if(chId==DEFAULT_CHANNEL)/*Strings, no channels*/
      {
	String chData=new String(data,0,1,chDataLen);
	Object[] args=new Object[1];
	args[0]=chData;
	for(int i=0;i<dataListScr.size();i++)
	  ((JSObject)dataListScr.elementAt(i)).call("Tango_receive",args);
      }
      else //if(chId>=0)/*bytes*/
      {
	String chName=String.valueOf(chId);
	byte[] chData=new byte[chDataLen];
	for(int i=0;i<chDataLen;i++)
	  chData[i]=data[1+i];
	Object[] args=new Object[2];
	args[0]=chName;args[1]=chData;
	for(int i=0;i<dataListScr.size();i++)
	  ((JSObject)dataListScr.elementAt(i)).call("Tango_channelReceive",args);
      }
    }// receive ____________________________________________________________


    public void masterChanged(boolean isMaster,String masterName)
    {
      for(int i=0;i<contListScr.size();i++)
      {
	Object args[]=new Object[2];
	args[0]=new Boolean(isMaster);
	args[1]=masterName;
	((JSObject)contListScr.elementAt(i)).call("Tango_masterChanged",args);
      }
    }// masterChanged ______________________________________________________


    public void participantJoined(String name)
    {
      for(int i=0;i<contListScr.size();i++)
      {
	Object args[]=new Object[1];
	args[0]=name;
	((JSObject)contListScr.elementAt(i)).call("Tango_participantJoined",args);
      }
    }// participantJoined __________________________________________________


    public void participantLeft(String name)
    {
      for(int i=0;i<contListScr.size();i++)
      {
	Object args[]=new Object[1];
	args[0]=name;
	((JSObject)contListScr.elementAt(i)).call("Tango_participantLeft",args);
      }
    }// participantLeft ____________________________________________________


    public void audioRequested(boolean r)
    {
      for(int i=0;i<contListScr.size();i++)
      {
	Object args[]=new Object[1];
	args[0]=new Boolean(r);
	((JSObject)contListScr.elementAt(i)).call("Tango_audioRequested",args);
      }
    }// audioRequested _____________________________________________________


    /**
     *@return thios user name (as logged to the system)*/
    public String getUserNameJS()
    {
      return getUserName();
    }// getUserNameJS ______________________________________________________


    /**
       *@param ss list to be printed
       *@param beg,end opening and closing braces
       *@param sep list separator
       *@return listing of the input array as a set
       (<beg><ss[0]><sep><ss[1]><sep>...<end>)*/
    private static String arrayToString(Object[] ss,String beg,String sep,String end)
    {
      int n;
      if(ss!=null)
	n=ss.length;
      else
	n=6;
      StringBuffer s=new StringBuffer(n*10);


      if(beg!=null)
	s.append(beg);

      if(ss!=null)
      {
	for(int i=0;i<n;i++)
	{
	  s.append(ss[i].toString());
	  if(i<(n-1))
	    s.append(sep);
	}
      }

      if(end!=null)
	s.append(end);


      return s.toString();
    }// toString ___________________________________________________________


    /**
     *@return Array object - array of Strings representing names
     of the users of this session*/
    public JSObject getParticipantNamesJS()
    {
      String[] names=getParticipantNames();
      Object eval=script.eval("new Array("+
			      arrayToString(names,"'","','","'")+");");
      //System.out.println(CL+".getParticipantNamesJS(): eval="+eval);
      return (JSObject)eval;
    }// getParticipantNamesJS ______________________________________________


    /**
     *@return the name of the master of this session*/
    public String getMasterNameJS()
    {
      return getMasterName();
    }// getMasterNameJS ____________________________________________________


    /**
     *@return Boolean object - true if this user
     is the master of this session*/
    public JSObject isMasterJS()
    {
      Object eval=script.eval("new Boolean("+(isMaster()?"'true'":"'false'")+");");
      //System.out.println(CL+".isMasterJS(): eval="+eval);
      return (JSObject)eval;
    }// isMasterJS _________________________________________________________


    /**
     *@return Boolean object - true if the audio resources
     can be used by this session*/
    public JSObject isAudioAvailableJS()
    {
      Object eval=script.eval("new Boolean("+(isAudioAvailable()?"'true'":"'false'")+");");
      //System.out.println(CL+".isMasterJS(): eval="+eval);
      return (JSObject)eval;
    }// isAudioAvailableJS _________________________________________________


    private String bytesToString(byte data[])
    {
      return new String(data,0);
    }// bytesToString ______________________________________________________


    private byte[] stringToBytes(String data)
    {
      int dataLen=data.length();
      byte[] bytes=new byte[dataLen];
      data.getBytes(0,dataLen,bytes,0);
      return bytes;
    }// stringToBytes ______________________________________________________


    /**Registers application and creates handle on Tango agent.
     *@param scr script/window to be registered - the top of this window
     will be closed upon the application termination*/
    public static TAgentJS createTAgentJS(JSObject scr)
    {
      try
      {
	return new TAgentJS(new TLAgentScript(scr),scr);
      }
      catch(TangoException e)
      {
	return null;
      }
    }// createTAgentJS _____________________________________________________


    public String toString()
    {
      return CL+"["+super.toString()+"]";
    }// toString ___________________________________________________________


}// TAgentJS ///////////////////////////////////////////////////////////////


