/***************************************************************************
  $Id: TLAgentApplet.java,v 1.8 1998/11/03 00:53:03 piotr Exp $
***************************************************************************/


package webwisdom.tango;


import java.applet.Applet;
import java.util.Vector;
import java.awt.*;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.structures.LocalBase;
import webwisdom.tango.interfaces.AppletBaseInterf;

import netscape.javascript.JSObject;


/**Tango system proxy for applets.
   This class may be used to create TAgent.
   *@see TAgent*/
public final class TLAgentApplet implements TLAgent,AppletBaseInterf
{
    private static final String CL="TLAgentApplet";


    private Applet applet=null;
    private AppletHandle handle=null;
    private TLListener lowlList;
 

    /**Registers an applet in the Tango system.
     *@param app applet to be registered
     *@exception TangoException if cannot get reference to Tango*/
    public TLAgentApplet(Applet app) throws TangoException
    {
      if(app==null)
	throw new IllegalArgumentException("null applet");
      applet=app;
      
      try
      {
	handle=register(this);
      }
      catch(UnsatisfiedLinkError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      catch(NoClassDefFoundError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      if(handle==null)
	throw new TangoException("Tango demon not found");
    }// TLAgentApplet _____________________________________________________


    public void exit()
    {
      //System.out.println(CL+".exit()");
      handle.remove();
      handle=null;
    }// exit ______________________________________________________________


    public void add(TLListener tl)
    {
      lowlList=tl;
    }// add _______________________________________________________________

  
    public final void send(AppEventMessage m)
    {
      handle.sendAppToOther(m);
    }// send ______________________________________________________________


    public void send(int userlist[],AppEventMessage m)
    {
      handle.sendAppToOther(userlist,m);
    }// send ______________________________________________________________


    public void receive(AppEventMessage m)
    {
      if(lowlList!=null)
	lowlList.receive(m);
    }// receive ___________________________________________________________


  public Object getJSWindow()
    {
      if(Const.inNetscape())
	{
	  try
	    {
	      JSObject window = (JSObject)((JSObject)JSObject.getWindow(applet));
         
    
	      JSObject parent=(JSObject)window.getMember("top");  		 	
	      String winName=(String)parent.getMember("name");
	      JSObject targetWindow=null;
	
	      System.out.println(CL + ": top window name " + winName);
	
	      if (winName.equals("TangoCa")) {

		System.out.println(CL + ": applet running in frame");
		//   name=(String)window.getMember("name");	
 
		targetWindow = window;
		// 	    inFrame = true;
        
	      } else {
    
		System.out.println(CL + ": applet running in window");
		targetWindow = parent;
		//    inFrame = false;	  
	      }
  
	      return targetWindow;  

  
	    }
	  catch(Exception e)
	    {
	      return null;
	    }
	}
      else
	{
	  Component c=applet;
	  while(c!=null && !(c instanceof Frame))
	    c=c.getParent();
	  return c;
	}
    }// getJSWindow _______________________________________________________

  static AppletHandle register(AppletBaseInterf base)
    {
      String name = null;  //name of the window or frame that contains register params
      boolean inFrame = false;
      String winName;
      int AID,SID,AT;  

      if(Const.inNetscape())
	{
	  netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserAccess");
	  JSObject window=(JSObject) base.getJSWindow();;
	  if(window==null)
	    return null;
	  JSObject parent=(JSObject)window.getMember("top");  		 	
	  winName=(String)parent.getMember("name");
	  name = (String)window.getMember("name");	
	}
      else
	{
	  name=base.getJSWindow().toString();
	  winName="";
	}
	
      System.out.println(CL + ": top window name " + winName);
	
      if (winName.equals("TangoCa"))
	{
	  System.out.println(CL + ": applet running in frame");
	  inFrame = true;
	}
      else
	{
	  System.out.println(CL + ": applet running in window");
	  inFrame = false;	  
	} 


      if((name==null)||(name.length()==0))
	{
	  System.err.println(CL+".register(): corrupted window name!");
	  return null;
	}
	
      AID=0;
      try
	{
	  int AIDend=name.indexOf('_');
	  int SIDend=name.indexOf('_',AIDend+1);
	  AID=Integer.parseInt(name.substring(3,AIDend));
	  SID=Integer.parseInt(name.substring(AIDend+1,SIDend));
	  AT=Integer.parseInt(name.substring(SIDend+1));
	  
	  System.out.println(CL+".register(): AID="+AID+",SID="+SID+",AT="+AT);
	}
      catch(Exception e)
	{
	  System.err.println(CL+".register(): corrupted window name!: "+e);
	  return null;
	}
      
      try
	{
	  AppletHandle h;  
	  
	  if (inFrame)
	    h=LocalBase.getLocalBase().reg.registerFrameApplet(base,AID,SID,AT, name);
	  else
	    h=LocalBase.getLocalBase().reg.registerApplet(base,AID,SID,AT);
	  
	  return h;
	}
      catch(Exception e)
	{
	  System.err.println(CL+".register(): plugin not found!");     
	  return null;
	}
      
    }// register __________________________________________________________


  public String toString()
    {
      return CL+"[applet="+applet+",handle="+handle+"]";
    }// toString ___________________________________________________________


}// TLAgentApplet /////////////////////////////////////////////////////////

  




