/****************************************************************************
  $Id: TangoAgent.java,v 1.6 1998/07/22 17:17:24 toma Exp $
****************************************************************************/


package webwisdom.tango;


import java.applet.*;
import java.util.Vector;

import webwisdom.tango.consts.Const;
import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.CAppAnswerEventMessage;
import webwisdom.tango.messages.CAppRequestEventMessage;

import netscape.javascript.JSObject;

import webwisdom.tango.msg.*;


class TangoAgentInitializer
{
    private static final String CL="TangoAgentInitializer";


    private TangoAgent agent;
    public boolean done=false;


    public TangoAgentInitializer(TangoAgent a)
    {
      agent=a;
    }// TangoAgentInitializer ______________________________________________


    public void init()
    {
      //System.out.println(CL+".init()");
      synchronized(this)
      {
	try
	{
	  while((agent.isMaster==null)||(agent.userName==null))
	    wait();
	}
	catch(InterruptedException e)
	{
	}
	done=true;
      }
      //System.out.println(CL+".init(): done");
    }// init _______________________________________________________________


    public synchronized void done()
    {
      //System.out.println(CL+".done()");
      notifyAll();
    }// done _______________________________________________________________


}// TangoAgentInitializer //////////////////////////////////////////////////


/**Java interface to Tango - Tango system proxy.*/
public abstract class TangoAgent
{
    private static final String CL="TangoAgent";


    protected TangoClient client=null;

    protected boolean isOwner=false;
    protected Boolean isMaster=null;
    protected boolean isReceiving=true,isSending=true;
    protected String userName=null,masterName=null,sessionName=null;
    protected String[] participants=null,activeUsers=null,localApps=null;

    private TangoAgentInitializer initializer=new TangoAgentInitializer(this);


    /**Registers an application in the Tango system
     *@param AID,SID,AT registration parameters
     *@exception Exception if cannot get reference to Tango*/
    protected TangoAgent(TangoClient clt/*,int AID,int SID,int AT*/)
      throws TangoException
    {
      if(clt==null)
	throw new RuntimeException("null client");
      client=clt;
    }// TangoAgent _________________________________________________________


    /**Exits from Tango. Cannot join Tango again.
       Must be placed in Applet.destroy().*/
    public abstract void exit();
    // exit ________________________________________________________________


    /**Allows/forbids sending messages to tango.
       Affects only send() and crossSend(), but not queryTango().
      *@param is whether send requests should be handled or ignored
      *@see #send
      *@see #crossSend*/
    public void setSending(boolean is)
    {
      isSending=is;
    }// setSending _________________________________________________________


    /**Allows/forbids receiving messages from tango.
      *@param ir whether receive() should be called when message arrives
      *@see #receive(byte[])*/
    public void setReceiving(boolean ir)
    {
      isReceiving=ir;
    }// setReceiving _______________________________________________________


    public final void receive(AppEventMessage m)
    {
      //System.out.println(CL+".receive("+m+")");


      int type=m.getType();
      if(type==Const.DATA)
      {
	if(isReceiving)
	{
	  int len=m.getLen();
	  byte[] buf=m.getData();
	  if(buf==null)
	  {
	    System.err.println(CL+".receive(DATA["+len+"]=null): ignored");
	    return;
	  }
	  else
	  {
	    //System.out.println(CL+".receive(DATA["+len+"])");
	    client.receive(buf);
	  }
	}
      }
      else if(type==Const.CONTROL)
      {
        CAppAnswerEventMessage mm=(CAppAnswerEventMessage)m;
        switch(mm.getAnswerType())
        {
          case CAConsts.ISMASTER:
	    isMaster=new Boolean(mm.isMaster());
	    System.out.println(CL+".receive(CONTROL.ISMASTER): "+isMaster);
	    initializer.done();
	    queryTango(CAConsts.MASTERNAME);
	    break;
	  case CAConsts.MASTERNAME:
	    masterName=mm.getMastername();
	    System.out.println(CL+".receive(CONTROL.MASTERNAME): "+masterName);
	    client.controlReceive(new TangoMsgMaster(isMaster.booleanValue(),masterName));
	    break;
	  case CAConsts.USERNAME:
	    userName=mm.getUsername();
	    System.out.println(CL+".receive(CONTROL.USERNAME): "+userName);
	    initializer.done();
	    break;
	  case CAConsts.PARTICIPANTS:
	    participants=toStringArray(mm.getParticipants());
	    System.out.println(CL+".receive(CONTROL.PARTICIPANTS): "+arrayToString(participants));
	    client.controlReceive(new TangoMsgParticipants(participants));
	    break;
	  case CAConsts.ACTIVEUSERS:
	    activeUsers=toStringArray(mm.getActiveUsers());
	    //System.out.println(CL+".receive(CONTROL.ACTIVEUSERS): "+arrayToString(activeUsers));
	    client.controlReceive(new TangoMsgUsers(activeUsers));
	    break;
	  case CAConsts.LOCALAPPS:
	    localApps=toStringArray(mm.getLocalApps());
	    //System.out.println(CL+".receive(CONTROL.LOCALAPPS): "+arrayToString(localApps));
	    client.controlReceive(new TangoMsgSessions(localApps));
	    break;
	  case CAConsts.DATAPASS:
	  {
	    if(isReceiving)
	    {
	      String sender=mm.getSender();
	      byte[] data=mm.getData();
	      //System.out.println(CL+".receive(CONTROL.DATAPASS)"+": sender="+sender+" data=bytes["+data.length+"]");
	      client.crossReceive(sender,data);
	    }
	    break;
	  }
	  case CAConsts.HOSTNAME:
	    break;
	  default:
	    System.err.println(CL+".receive(CONTROL."+mm.getAnswerType()+")"+": ignored");
	}
      }
      else
	System.err.println(CL+".receive("+type+"): ignored");
    }// receive ____________________________________________________________


    /**Puts elements of vector into array as strings.
       *@param v vector to be converted to array
       *@return array of strings representing vector elements*/
    private static String[] toStringArray(Vector v)
    {
      int n=v.size();
      String[] a=new String[n];
      for(int i=0;i<n;i++)
	a[i]=v.elementAt(i).toString();
      return a;
    }// toStringArray ______________________________________________________


    /**Array printing utility.
     *@param ss array to be converted to String
     *@param beg beginning of the string
     *@param sep separator between elements of array
     *@param end end of the string*/
    public static String arrayToString(Object[] ss,
				       String beg,String sep,String end)
    {
      int n;
      if(ss!=null)
	n=ss.length;
      else
	n=6;
      StringBuffer s=new StringBuffer(n*10);


      if(beg!=null)
	s.append(beg);

      if(ss!=null)
      {
	for(int i=0;i<n;i++)
	{
	  s.append(ss[i].toString());
	  if(i<(n-1))
	    s.append(sep);
	}
      }

      if(end!=null)
	s.append(end);


      return s.toString();
    }// toString ___________________________________________________________


    /**Equivalent of arrayToString(ss,null", ",null).
     *@see #arrayToString(Object[],String,String,String)*/
    public static String arrayToString(Object[] ss)
    {
      return arrayToString(ss,"{",",","}");
    }// arrayToString ______________________________________________________


    /**Sends queries to CA.*/
    protected void queryTango(byte q)
    {
      byte[] bytes=new byte[1];
      bytes[0]=q; 
      CAppRequestEventMessage m=new CAppRequestEventMessage(bytes);
      send(m);
    }// queryTango _________________________________________________________


    public boolean isMaster()
    {
      if(!initializer.done)
	initializer.init();
      return isMaster.booleanValue();
    }// isMaster ___________________________________________________________


    public String getUserName()
    {
      if(!initializer.done)
	initializer.init();
      return userName;
    }// getUserName ________________________________________________________


    public String getSessionName()
    {
//       if(!initializer.done)
// 	initializer.init();
      return sessionName;
    }// getUserName ________________________________________________________


    /**Sends messages in the low level format compatible with AppletBase.
      *@param m message*/
    protected abstract void send(AppEventMessage m);
    // send ________________________________________________________________


    /**Sends data to the same session.
      *@param bytes exchanged data*/
    public void send(byte[] bytes)
    {
      //System.out.println(CL+".send(byte["+bytes.length+"])");
      if(isSending)
	send(new AppEventMessage(Const.DATA,bytes));
    }// send _______________________________________________________________


    /**Sends data to a different session.
      *@param appName destination application
      *@param bytes exchanged data
      *@exception TangoException if invalid session name*/
    public void crossSend(String appName,byte[] bytes)
      throws TangoException
    {
      //System.out.println(CL+".crossSend(\""+appName+"\",byte["+bytes.length+"])");
      if(isSending)
      {
	byte[] b=CAppRequestEventMessage.prepareData(appName,bytes);
	CAppRequestEventMessage m=new CAppRequestEventMessage(b);
	send(m);
      }
    }// crossSend __________________________________________________________


    public String toString()
    {
      return CL+"[userName="+userName+
	",isOwner="+isOwner+",isMaster="+isMaster+
	",isSending="+isSending+",isReceiving="+isReceiving+"]";
    }// toString ___________________________________________________________


}// TangoAgent /////////////////////////////////////////////////////////////


