/****************************************************************************
  $Id: TangoAppletAgent.java,v 1.7 1998/11/03 00:53:03 piotr Exp $
****************************************************************************/


package webwisdom.tango;


import java.applet.Applet;
import java.util.Vector;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.interfaces.AppletBaseInterf;
import webwisdom.tango.interfaces.AppletBase;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.LocalBase;

import netscape.javascript.JSObject;


/**Java interface to Tango.*/
public final class TangoAppletAgent extends TangoAgent
  implements AppletBaseInterf
{
    private static final String CL="TangoAppletAgent";


    private Applet applet=null;
    private AppletHandle appletHandle=null;


    /**Registers an applet in the Tango system
     *@param app applet to be registered
     *@exception TangoException if cannot get reference to Tango*/
    public TangoAppletAgent(TangoClient clt,Applet app)
      throws TangoException
    {
      super(clt);

      if(app==null)
	throw new RuntimeException("null applet");
      applet=app;


      try
      {
	appletHandle=register(this,applet,0,0,0);
      }
      catch(UnsatisfiedLinkError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      if(appletHandle==null)
	throw new TangoException("Tango demon not found");


      queryTango(CAConsts.ISMASTER);//called automatically
      queryTango(CAConsts.USERNAME);

      //queryTango(CAConsts.PARTICIPANTS);//called automatically
      queryTango(CAConsts.ACTIVEUSERS);
      queryTango(CAConsts.LOCALAPPS);

      System.out.println(CL+".TangoAppletAgent(): calling isMaster()...");
      if(isMaster())
	isOwner=true;
      System.out.println(CL+".TangoAppletAgent(): calling isMaster() returned");
    }// TangoAppletAgent ___________________________________________________


    private static AppletHandle register(AppletBaseInterf base,
					 Applet applet,
					 int AID,int SID,int AT)
    {
      netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserAccess");

      if(applet!=null)
      {
	JSObject window=(JSObject)JSObject.getWindow(applet);
	if(window==null)
	{
	  System.err.println(CL+".register("+applet+"): null window!");
	  return null;
	}
	JSObject parent=(JSObject)window.getMember("top");  		 
	String name=(String)parent.getMember("name");	
	System.out.println(CL+".register(Applet): window="+window+",parent="+parent+",name=\""+name+"\"");
	if((name==null)||(name.length()==0))
	{
	  System.err.println(CL+".register(): window \"name\" not found: "+
			     "switching to standalone mode");
	  return null;
	}
	//System.out.println(CL+".register(): name=\""+name+"\"");
     
	AID=0;
	try
	{
	  int AIDend=name.indexOf('_');
	  int SIDend=name.indexOf('_',AIDend+1);
	  AID=Integer.parseInt(name.substring(3,AIDend));
	  SID=Integer.parseInt(name.substring(AIDend+1,SIDend));
	  AT=Integer.parseInt(name.substring(SIDend+1));
       
	  System.out.println(CL+".register(): AID="+AID+",SID="+SID+",AT="+AT);
	}
	catch(Exception e)
	{
	  System.err.println(CL+".register(): AID not found!: "+
			     "switching to standalone mode");     
	  //e.printStackTrace();
	  return null;
	}
      }


      LocalBase plugin=LocalBase.getLocalBase();
      if(plugin==null)
      {
	System.err.println(CL+".register(): plugin not found!: "+
			   "switching to standalone mode");     
	return null;
      }
      AppletHandle h=plugin.reg.registerApplet(base,AID,SID,AT);
      return h;
    }// register ___________________________________________________________


//     /**Registers an application in the Tango system
//      *@param AID,SID,AT registration parameters
//      *@exception Exception if cannot get reference to Tango*/
//     public TangoAppletAgent(TangoClient clt,int AID,int SID,int AT)
//       throws TangoException
//     {
//       super(clt);


//       try
//       {
// 	appletHandle=AppletBase.register(this,null,AID,SID,AT);
//       }
//       catch(UnsatisfiedLinkError e)
//       {
// 	throw new TangoException("Not in Netscape");	
//       }
//       if(appletHandle==null)
// 	throw new TangoException("Tango demon not found");


//       //queryTango(CAConsts.ISMASTER);
//       if(isMaster)
// 	isOwner=true;
//       //queryTango(CAConsts.USERNAME);
//       queryTango(CAConsts.PARTICIPANTS);
//       queryTango(CAConsts.ACTIVEUSERS);
//       queryTango(CAConsts.LOCALAPPS);
//     }// TangoAppletAgent ___________________________________________________


    /**Exits from Tango. Cannot join Tango again.
       Must be placed in Applet.destroy().*/
    public void exit()
    {
      //System.out.println(CL+".destroy()");
      appletHandle.remove();
      appletHandle=null;
    }// exit _______________________________________________________________


    protected final void send(AppEventMessage m)
    {
      appletHandle.sendAppToOther(m);
    }// send _______________________________________________________________


//     public static JSObject getJSWindow(Applet applet)
//     {
//       JSObject window=(JSObject)JSObject.getWindow(applet);
//       if(window==null)
//       {
// 	System.err.println(CL+".getJSWindow("+applet+"): unable to get reference to window!");
// 	window=null;
//       }
//       return window;
//     }// getJSWindow ________________________________________________________


    public Object getJSWindow()
    {
      return (JSObject)JSObject.getWindow(applet);
    }// getJSWindow ________________________________________________________


    public String toString()
    {
      return CL+"["+super.toString()+
	",applet="+applet+",appletHandle="+appletHandle+"]";
    }// toString ___________________________________________________________


}// TangoAppletAgent ///////////////////////////////////////////////////////


