/****************************************************************************
  $Id: TangoApplicationAgent.java,v 1.6 1998/07/22 17:17:26 toma Exp $
****************************************************************************/


package webwisdom.tango;


import java.net.*;
import java.io.*;

import java.util.Vector;

import webwisdom.tango.consts.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.interfaces.*;


class TangoApplicationAgentReceiver extends Thread
{
    private static final String CL="TangoApplicationAgentReceiver";


    DataInputStream inStream;
    TangoClient client;


    public TangoApplicationAgentReceiver(DataInputStream is,TangoClient tc)
    {
      inStream=is;
      client=tc;
    }// TangoApplicationAgentReceiver ______________________________________


    public String toString()
    {
	return CL+"[]";
    }// toString ___________________________________________________________


    public void run()
    {
      try
      {
	do
	{
	  int type=inStream.readInt();
	  int packetLen=inStream.readInt();
	  byte[] packetData=new byte[packetLen];
	  inStream.readFully(packetData);
	  if(type==TangoApplicationAgent.DATA)
	  {
	    client.receive(packetData);
	  }
	  else if(type==TangoApplicationAgent.CONTROL)
	  {
	    switch(packetData[0])
	    {
	      case 62://but connection broken first
		System.exit(0);
	      default:
		;
	    }
	  }
	  //System.out.println(CL+".< "+new String(packetData,0));
	}
	while(true);
      }
      catch(EOFException e)
      {
	//System.err.println(CL+": Server disconnected!");
	System.exit(0);
      }
      catch(IOException e)
      {
	System.err.println(CL+": "+e);
      }
    }// run ________________________________________________________________


}// TangoApplicationAgentReceiver //////////////////////////////////////////




/**Java interface to Tango.
 *@author Tomasz Major
 *@see Event.c*/
public final class TangoApplicationAgent extends TangoAgent
{
    private static final String CL="TangoApplicationAgent";


    static final int CONTROL=30;
    static final int DATA=31;


    private Socket socket;
    private DataOutputStream outStream;
    private DataInputStream inStream;

    private TangoApplicationAgentReceiver receiverThread;


    /**Registers an applet in the Tango system
     *@param app applet to be registered
     *@exception TangoException if cannot get reference to Tango
     *@exception IOException if problems with sockets*/
    public TangoApplicationAgent(TangoClient clt,String[] args)
      throws TangoException,IOException
    {
      super(clt);
      register(args);
    }// TangoApplicationAgent ______________________________________________


    private void register(int AID,int SID,int AT,int port)
      throws TangoException,IOException
    {
      try
      {
	InetAddress host=InetAddress.getLocalHost();
	socket=new Socket(host,port); 
	outStream=new DataOutputStream(socket.getOutputStream());
	inStream=new DataInputStream(socket.getInputStream());
	/*System.out.println(CL+": connected to: "+socketToString(socket)+
	  " (localPort="+socket.getLocalPort()+")");*/
      }
      catch(IOException e)
      {
	throw new TangoException("port "+port+" unreachable");
      }

      receiverThread=new TangoApplicationAgentReceiver(inStream,client);
      receiverThread.start();
      (Thread.currentThread()).setPriority(Thread.MIN_PRIORITY);


      outStream.writeInt(AID);
      outStream.writeInt(SID);
      outStream.writeInt(AT);
      outStream.flush();
    }// register ___________________________________________________________


    private void register(String[] args)
      throws TangoException,IOException
    {
      int AID=0,SID,AT;
      int port;

      try
      {
	AID=Integer.valueOf(args[1]).intValue();
	SID=Integer.valueOf(args[2]).intValue();
	AT=Integer.valueOf(args[3]).intValue();
	port=Integer.valueOf(args[4]).intValue();

// 	for(int i=0;i<args.length;)
// 	{
// 	  String pn=args[i++],p=args[i++];
// 	  if(pn.equals("-AID"))
// 	  {
// 	    AID=Integer.valueOf(p).intValue();
// 	  }
// 	  else if(pn.equals("-SID"))
// 	  {
// 	    SID=Integer.valueOf(p).intValue();
// 	  }
// 	  else if(pn.equals("-AT"))
// 	  {
// 	    AT=Integer.valueOf(p).intValue();
// 	  }
// 	  else if(pn.equals("-port"))
// 	  {
// 	    port=Integer.valueOf(p).intValue();
// 	  }
// 	}
      }
      catch(NumberFormatException e)
      {
	throw new TangoException("wrong params");
      }
      register(AID,SID,AT,port);
    }// register __________________________________________________________


    public void exit()
    {
      System.exit(0);
    }// exit _______________________________________________________________


    protected final void send(AppEventMessage m)
    {
      ;
    }// send _______________________________________________________________


    public void send(byte[] b)
    {
      try
      {
	outStream.writeInt(DATA);
	outStream.writeInt(b.length);
	outStream.write(b);
	outStream.flush();
	//System.out.println(CL+".send(byte["+b.length+"])");
      }
      catch(IOException e)
      {
	System.err.println(CL+".send(): "+e);
      }
    }// send _______________________________________________________________


    public String toString()
    {
      return CL+"[isMaster="+isMaster+
	",isSending="+isSending+",isReceiving="+isReceiving+"]";
    }// toString ___________________________________________________________


}// TangoApplicationAgent //////////////////////////////////////////////////


