// $Id: Pipe.java,v 1.3 1998/09/22 18:34:17 beca Exp $
/*
File: Pipe.java
Component: Plugin
Date            Author        Changes
Aug 05 98       Lukasz Beca   Created
*/

package webwisdom.tango.beans;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.applet.*;
import java.io.*;


public abstract class Pipe extends Component implements java.io.Serializable
{
	public Pipe()
	{
		super();
	//	setSize(50,40);
    	setFont(new Font("Dialog", Font.PLAIN, 12));
		sizeToFit();

		if (Beans.isDesignTime()) {
		    setVisible(true);
		} else {
		    setVisible(false);		 
		}
	}

	public void paint(Graphics context)
	{
				
		int width = getSize().width;
    	int height = getSize().height;
	    
	    FontMetrics fm = getFontMetrics(getFont());
	    
		context.setColor(componentColor);
		context.fill3DRect(0,0, width, height, true);
		context.setColor(textColor);
		context.drawString("TANGO", (width - fm.stringWidth("TANGO")) / 2, (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
		context.setColor(componentColor);
		
	}

public Dimension getPreferredSize() {
	FontMetrics fm = getFontMetrics(getFont());
	//System.out.println("current size " + getSize().width + " " + getSize().height); 
	//System.out.println("preferred size " + (fm.stringWidth("TANGO") + 2* TEXT_XPAD) + " " + (fm.getMaxAscent() + fm.getMaxDescent() + 2*TEXT_YPAD));
	
	return new Dimension(fm.stringWidth("TANGO") + 2* TEXT_XPAD, 
			     fm.getMaxAscent() + fm.getMaxDescent() + 2*TEXT_YPAD);
    }

    /**
     * @deprecated provided for backward compatibility with old layout managers.
     */
    public Dimension preferredSize() {
        
       // System.out.println("preferred size called " + getSize().width + " " + getSize().height); 
	return getPreferredSize();
    }

    public Dimension getMinimumSize() {
	return getPreferredSize();
    }

    /**
     * @deprecated provided for backward compatibility with old layout managers.
     */
    public Dimension minimumSize() {
	return getMinimumSize();
    }

    private void sizeToFit() {
	
    	Dimension pd = getPreferredSize();
    	setSize(pd.width, pd.height);
    	//System.out.println("sizetofit size " + getSize().width + " " + getSize().height); 

    	Component p = getParent();
    	if (p != null) {
	        p.invalidate();
	        p.validate();
	     }
	
    }

	public synchronized Color getComponentColor()
	{
		return componentColor;
	}

	public void setComponentColor (Color newColor)
	{
		componentColor = newColor;
		repaint();
	}

	
    public synchronized void addPipeConnectListener(PipeConnectListener tel) {
    
        PipeConnectEvent cre = new PipeConnectEvent(this, false);
        tel.pipeReady(cre); 
    }

    public synchronized void removePipeConnectListener(PipeConnectListener tel) {

    }
    
    public synchronized void setControl(int id, TangoBean c) {
           
        ctr = c;
        pipeID = id;
    }
    
    public void sendDataAsObject(Object toTango) throws IOException, TangoBeansException {
        
        if (ctr == null) throw new PipeNotConnectedException();
        
//        if (ctr != null) {
        
            ctr.sendToTango(pipeID, toTango);   
            
  //      } else {
            
      //      throw new PipeNotConnectedException();
            //System.out.println("Pipe: pipe not connected to TangoBean");   
    //    }
        
    }
    
    public void selectiveSendDataAsObject(String[] participants, Object toTango) throws IOException, TangoBeansException {
        
        if (ctr == null) throw new PipeNotConnectedException();
        if (participants.length == 0) throw new NoRecipientsSelectedException();
        
      //  if (ctr != null) {
        
            ctr.selectiveSendToTango(participants, pipeID, toTango);               
      //  } else {
            
           // throw new PipeNotConnectedException();
            //System.out.println("Pipe: pipe not connected to TangoBean");   
       // }        
    }


    public abstract void receiveData(Object fromTango);

	protected Color componentColor = Color.green;
	protected Color textColor = Color.black;
	protected TangoBean ctr;
	private int pipeID;
	static final int TEXT_XPAD = 10;
    static final int TEXT_YPAD = 10;

}