/****************************************************************************
  $Id: FakeTangoServer.java,v 1.3 1998/07/10 18:54:06 remek Exp $
  PURPOSE: easy testing of Tango applets
****************************************************************************/


package webwisdom.tango.fake;


import java.net.*;
import java.io.*;
import java.util.Vector;

//import tm.std.*;
import java.awt.*;
//import tm.awt.*;


class ClientThread extends Thread
{	
    private static final String CL="ClientThread";
    private static final String CL2="FTS";


    FakeTangoServer server;

    Socket socket;
    private String host;
    private int port;
    DataInputStream inStream;
    DataOutputStream outStream;


    public ClientThread(FakeTangoServer s,Socket so) throws IOException
    {
      server=s;
      socket=so;
      host=socket.getInetAddress().getHostName();
      port=socket.getPort();
      inStream=new DataInputStream(socket.getInputStream());
      outStream=new DataOutputStream(socket.getOutputStream());
      System.out.println(CL2+"."+this+": accepted");
    }// ClientThread _______________________________________________________


    public void run()
    {
      do
      {
	int packetLen=0;
	byte[] packetData=null;
	try
	{
	  packetLen=inStream.readInt();
	  packetData=new byte[packetLen];
	  inStream.readFully(packetData);
	  //System.out.println(CL+".Received: "+packetLen);
	}
	catch(EOFException e)
	{
	  System.err.println(CL2+"."+this+": disconnecting");
	  synchronized(server.clients)
	  {
	    server.clients.removeElement(this);
	  }
	  stop2();
	  return;
	}
	catch(IOException e)
	{
	  System.err.println(CL2+"."+this+": !ignoring: "+e);
	  //e.printStackTrace();
	  continue;
	}
	
	synchronized(server.clients)
	{
	  for(int i=0;i<server.clients.size();i++)
	  {
	    ClientThread cl=(ClientThread)server.clients.elementAt(i);
	    //System.out.println(CL+".Serving["+i+"]: "+cl);
	    if(cl==this)
	      continue;
	    try
	    {
	      DataOutputStream s=cl.outStream;
	      s.writeInt(packetLen);
	      s.write(packetData);
	      s.flush();
	      //System.out.println(CL+".Sent["+i+"]: "+packetLen);
	    }
	    catch(IOException e)
	    {
	      System.err.println(CL2+"."+this+": disconnecting "+cl);
	      cl.stop2();
	      server.clients.removeElement(cl);
	      //server.removeClient(cl);
	      i--;
	    }
	  }
	}
      }
      while(true);
    }// run ________________________________________________________________


    public void stop2()
    {
      try
      {
	inStream.close();
	outStream.close();
	socket.close();
      }
      catch(IOException e)
      {
      }
      stop();
    }// stop2 ______________________________________________________________


    public String toString()
    {
      return "["+host+":"+port+"]";
    }// toString ___________________________________________________________


    public static String socketToString(Socket s)
    {
      return "["+s.getInetAddress().getHostName()+":"+s.getPort()+"]";
    }// socketToString _____________________________________________________


}// ClientThread ///////////////////////////////////////////////////////////




/**Simple server broadcasting messages from clients to all other clients.
   Clients are instances of FakeTangoAgent.
   Run with parameter being a port number.
   Console communicates have two formats:<ul>
   <li><b>FTS: </b><i>message</i>; or
   <li><b>FTS.[</b><i>clientThread</i><b>]: </b><i>message</i>.</ul>
   *@see FakeTangoAgent
   *@author Tomasz Major*/
public class FakeTangoServer
{
    private static final String CL="FakeTangoServer";
    private static final String CL2="FTS";


    ServerSocket server;
    static final int MAX_CLIENTS=10;
    Vector clients=new Vector(MAX_CLIENTS,1);


    /**Anchors at the given port.
     *@param port port at which the server is to listen for clients
     *@exception IOException if cannot start service at given port*/
    public FakeTangoServer(int port) throws IOException
    {
      server=new ServerSocket(port);
      port=server.getLocalPort();
      System.out.println(CL2+": started on port "+port);
    }// FakeTangoServer ____________________________________________________


    /**Starts the server.
       Servers accepts connections from clients
       and creates new thread for each client.
       *@exception IOException not caught system exceptions*/
    public void run() throws IOException
    {
      do
      {
	Socket socket=server.accept();
	if(clients.size()>=MAX_CLIENTS)
	{
	  System.err.println(CL2+": rejecting client "+
			     ClientThread.socketToString(socket));
	  socket.close();
	  continue;
	}
	ClientThread cl=new ClientThread(this,socket);
	cl.setPriority((Thread.currentThread()).getPriority()+1);
	synchronized(clients)
	{
	  cl.start();
	  clients.addElement(cl);
	}
      }
      while(true);
    }// run ________________________________________________________________


    public String toString()
    {
	return CL+"[#="+clients.size()+"]";
    }// toString ___________________________________________________________


    /**Creates and starts the server at the port given the first parameter.*/
    public static void main(String[] args)
    {
      int port=-1;
      try
      {
	port=Integer.valueOf(args[0]).intValue();
      }
      catch(NumberFormatException e)
      {
	System.err.println(CL+".main(): Usage: main(port)");
	System.exit(2);
      }

      FakeTangoServer s=null;
      try
      {
	s=new FakeTangoServer(port);
      }
      catch(IOException e)
      {
	System.err.println(CL+".main(): Cannot start server at port "+port+": "+e);
	System.exit(1);
      }

      try
      {
	s.run();
      }
      catch(IOException e)
      {
	System.err.println(CL+".main(): I/O error: "+e);
	System.exit(1);
      }
    }// main _______________________________________________________________


}// FakeTangoServer ////////////////////////////////////////////////////////


