// $Id: AppEventMessage.java,v 1.5 1998/07/10 20:15:19 beca Exp $
// $Id: AppEventMessage.java,v 1.5 1998/07/10 20:15:19 beca Exp $
// $Id: AppEventMessage.java,v 1.5 1998/07/10 20:15:19 beca Exp $
/*
File: AppEventMessage.java
Component: Plugin
Date		Author		Changes
Sept 6 96	Lukasz Beca	Created

*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;



/**
 * A class representing message that is send and received by applets and application 
 * 
 *@author Lukasz Beca
 *@see Interfaces in CIV Communication System documentation
 **/
public class AppEventMessage extends Message
{
    private static final String CL="AppEventMessage";


    private int len;
    private byte data[];


    /**
     *@param typepar type of message
     <ul>
     <li>if type is set to Const.DATA,
     message will be sent to all applications in the same session;
     <li>if type is set to Const.CONTROL,
     message will be sent to local control application.
     </ul>
     *@param datapar table containing information
     *@see Const*/
    public AppEventMessage(int typepar,byte datapar[])
    {
      type=typepar;
      len=datapar.length;
      data=new byte[len];
      for(int i=0;i<len;i++)
      {
	data[i]=datapar[i];
      }
    }// AppEventMessage ____________________________________________________


    /**Creates a new EVENT message from the stream
     *@param instream message data*/
    AppEventMessage(DataInputStream instream) throws IOException
    {
      type=instream.readInt();
      if((type==Const.DATA)||(type==Const.CONTROL))
      {
	len=instream.readInt();
	data=new byte[len];         
	instream.readFully(data,0,len);
      }
      else if(type==Const.PING)
      {
	len=instream.readInt();
	data=new byte[len];         
	instream.readFully(data,0,len);
      }
      else
      {
	System.err.println(CL+".AppEventMessage(): type="+type+
			   " not supported!: creating null message");
	type=Const.UNDEFINED;
	len=0;
	data=null;
      }
    }// AppEventMessage ____________________________________________________


    /**Writes EVENT message to the output stream
     *@param outstream output stream*/
    void send(DataOutputStream outstream) throws IOException
    {
      //System.out.println(CL+".send(): "+this);
      outstream.writeInt(type);
      outstream.writeInt(len);
      outstream.write(data,0,len);
      outstream.flush();
    }// send _______________________________________________________________
  

    /**
     *@return returns length of table containing information*/
    public int getLen()
    {
      return len;
    }// getLen _____________________________________________________________   


    /**
     *@return returns reference to the table with information*/
    public byte[] getData()
    {
      return data;
    }// getDate ____________________________________________________________


    public String toString()
    {
      return CL+"[type="+type+",byte["+len+"]]";
    }// toString ___________________________________________________________


}// AppEventMessage ////////////////////////////////////////////////////////


