// $Id: EventMessage.java,v 1.4 1998/07/10 20:15:24 beca Exp $
/*
File: EventMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
Nov 19 96       Lukasz Beca     Full read added
*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;


/**
 * A class representing message EVENT
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class EventMessage extends Message
{
    private static final String CL="EventMessage";


    private int AID;
    private int len;
    private byte data[];


  /**
   * Creates a new EVENT message
   *@param AIDparam application number
   *@param datapar table containing information
   **/

  public EventMessage(int AIDpar, byte datapar[]) {

    int i;

   AID = AIDpar;
   len = datapar.length;
   data = new byte[len];

   for (i = 0; i < len; i ++) {

     data[i] = datapar[i];

   }
   
   type = Const.EVENT;
  }

   /**
   * Creates a new EVENT message, information is read from the input stream
   *@param instream  input stream
   **/

  EventMessage(DataInputStream instream) throws IOException {
    

 
    AID = instream.readInt();
    len = instream.readInt();
    data = new byte[len];
    instream.readFully(data, 0, len);   

    type = Const.EVENT;
    
  }

   /**
   * Writes EVENT message to the output stream
   *@param outstream  output stream
   **/


    void send(DataOutputStream outstream) throws IOException
    {
      // System.out.println("Sending type EVENT");
      outstream.writeInt(Const.EVENT);
      outstream.writeInt(AID);
      outstream.writeInt(len);
      outstream.write(data, 0, len);
      outstream.flush();
    }
  
  /**
   * Returns application number
   **/


  public int getAID() {
    
    return AID;

  }  
  /**
   * Returns length of table containing information
   **/

  public int getLen() {
    
    return len;

  }   

  /**
   * Returns reference to the table with information
   **/

  
  public byte[] getData() {
    
    return data;
  }

    public String toString()
    {
      return CL+"[AID="+AID+",byte["+len+"]]";
    }// toString __________________________________________________________


}// EventMessage //////////////////////////////////////////////////////////


