// $Id: JansMessage.java,v 1.4 1998/07/10 20:15:27 beca Exp $
/*
File: JansMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 24 96	Lukasz Beca	Created
*/

package webwisdom.tango.messages;

import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;


/**
 * A class representing message JANS
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/


public class JansMessage extends Message
{
    private static final String CL="JansMessage";


    private int answer;
    private int UID;
    private int SID;
    private int AT;


   /**
   * Creates a new JANS message
   *@param ATpar application type
   *@param SIDpar session number
   *@param UIDpar user number
   *@param answerpar answer: Const.YES or Const.NO 
   *@see Const
   **/
  


  public JansMessage(int ATpar, int SIDpar, int UIDpar, int answerpar) {

    AT = ATpar;
   SID = SIDpar;
   UID = UIDpar;
   answer = answerpar;

   type = Const.JANS;
  
  }


   /**
   * Creates a new JANS message, information is read from the input stream
   *@param instream  input stream
   **/


  JansMessage(DataInputStream instream) throws IOException {
    

   
    AT = instream.readInt();
    SID= instream.readInt();     
    UID = instream.readInt();
    answer = instream.readInt();
       
    type = Const.JANS;
    
  }
   /**
   * Writes JANS message to the output stream
   *@param outstream  output stream
   **/

  void send(DataOutputStream outstream) throws IOException {
  

    
    //System.out.println("Sending type " + Const.JANS);
    outstream.writeInt(Const.JANS);
    //System.out.println("Sending AT " + AT);
    outstream.writeInt(AT);
    //System.out.println("Sending SID " + SID);
    outstream.writeInt(SID);
    //System.out.println("Sending UID " + UID);
    outstream.writeInt(UID);
    //System.out.println("Sending answer " + answer);
    outstream.writeInt(answer);

    outstream.flush();
  

  }
  

   /**
   * Returns user number
   **/

    public int getUID()
    {
      return UID;
    }  


   /**
   * Returns session number
   **/

  public int getSID() {
   
    return SID;
  }
  
   /**
   * Returns application type
   **/

  public int getAT() {
   
    return AT;
  }
   /**
   * Returns answer
   **/

  public int getAnswer() {
   
    return answer;
  }
  
    public String toString()
    {
      return CL+"[AT="+AT+",SID="+SID+",UID="+UID+",answer="+answer+"]";
    }// toString __________________________________________________________


}// JansMessage ///////////////////////////////////////////////////////////

