// $Id: MansMessage.java,v 1.4 1998/07/10 20:15:31 beca Exp $
/*
File: MansMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;


/**
 * A class representing MANS message
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class MansMessage extends Message
{
    private static final String CL="MansMessage";


    private int answer;
    private int UID;
    private int SID;


/**
   * Creates a new MANS message
   *@param SIDpar session number
   *@param UIDpar user number
   *@param answerpar answer: Const.YES or Const.NO 
   *@see Const
   **/

  public MansMessage(int SIDpar, int UIDpar, int answerpar) {

   SID = SIDpar;
   UID = UIDpar;
   answer = answerpar;

   type = Const.MANS;
  
  }

 /**
   * Creates a new MANS message, information is read from the input stream
   *@param instream  input stream
   **/


  MansMessage(DataInputStream instream) throws IOException {
    

   

    SID= instream.readInt();     
    UID = instream.readInt();
    answer = instream.readInt();

    type = Const.MANS;
    
  }

 /**
   * Writes MANS message to the output stream
   *@param outstream  output stream
   **/


  void send(DataOutputStream outstream) throws IOException {
  

    //System.out.println("Sending type " + Const.MANS);
    outstream.writeInt(Const.MANS);
    //System.out.println("Sending SID " + SID);
    outstream.writeInt(SID);
    //System.out.println("Sending UID " + UID);
    outstream.writeInt(UID);
    //System.out.println("Sending answer " + answer);
    outstream.writeInt(answer);

    outstream.flush();

  }

 /**
   * Returns user number
   **/
  

  public int getUID() {
       
    return UID;

  }  

 /**
   * Returns session number
   **/  

  public int getSID() {
   
    return SID;
  }

  /**
   * Returns answer
   **/


  public int getAnswer() {
   
    return answer;
  }
  
    public String toString()
    {
      return CL+"[UID="+UID+",SID="+SID+",answ="+answer+"]";
    }// toString __________________________________________________________


}// MansMessage ///////////////////////////////////////////////////////////


