// $Id: Net.java,v 1.5 1998/07/10 20:15:34 beca Exp $
/*
File: Net.java
Component: Server, Plugin
Date		Author		Changes
Aug 24 96	Lukasz Beca	Created
*/


package webwisdom.tango.messages;


import java.io.*;
import java.net.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.consts.Const;


/**Objects of this class send and receive messages using network connection
 *@author Lukasz Beca*/
public class Net
{
    private static final String CL="Net";


    Socket s;
    DataInputStream in;
    DataOutputStream out;
    String host;
    String IP;


    /**Creates a new Net object
     *@param sparam socket*/
    public Net(Socket sparam)
    {
      s=sparam;
      try
      {
	in = new DataInputStream(s.getInputStream());
	out = new DataOutputStream(s.getOutputStream());
	host = new String (s.getInetAddress().getHostName());
	IP = new String(s.getInetAddress().getHostAddress());
      }
      catch(Exception e)
      {
	System.err.println(CL+" exception!: "+e);
	e.printStackTrace();	
	System.exit(1);
      }
    }// Net ________________________________________________________________


    /**Sends message 
     *@param m message
     *@exception IOException*/
    public void send(Message m) throws IOException
    {
      m.send(out);
    }// send _______________________________________________________________


    /**Returns message
     *@return message
     *@exception IOException*/
    public Message receive() throws IOException
    {
      int type=in.readInt();
      //System.out.println(CL+".receive(): type="+type);
      if(type==Const.EVENT)
      {
      	return new EventMessage(in);
      }
      else
      {
	switch(type)
	{
	  //server part 
	  case Const.LOGIN:
	    //System.out.println(CL+".receive(): LOGIN");
	    return new LoginMessage(in);
	  case Const.LOGOUT:
	    return new LogoutMessage(in);
	  case Const.JOIN:
	    return new JoinMessage(in);
	  case Const.STARTED:
	    return new StartedMessage(in);
	  case Const.FINISH:
	    return new FinishMessage(in);
	  case Const.MASTER:
	    return new MasterMessage(in);
	  case Const.MANS:
	    return new MansMessage(in);
	  case Const.RJOIN:
	    return new RjoinMessage(in);
	  case Const.RANS:
	    return new RansMessage(in);
	  case Const.JANS:
	    return new JansMessage(in);
	  case Const.GROUP_EVENT:
	    return new GroupEventMessage(in);
	  //demon part
	  case Const.JASK:
	    return new JaskMessage(in);
	  case Const.LOGACC:
	    return new LogaccMessage(in);
	  case Const.SUPD:
	    return new SupdMessage(in);
	  case Const.COMMAND:
	    return new CommandMessage(in);
	  case Const.MASK:
	    return new MaskMessage(in);
	  case Const.RASK:
	    return new RaskMessage(in);
	  case Const.PING:
	    return new PingMessage();
	  default:
	    System.err.println(CL+".receive(): type not implemented!: "+type);
	    return null;
	}
      }	
    }// receive ____________________________________________________________


    /**Sends message and user number 
     *@param UID user number
     *@param m message
     *@exception IOException
     *@see class IntUID*/
    public void sendUID(IntUID UID,Message m) throws IOException
    {
      out.writeInt(UID.getUID());        
      m.send(out);
    }// sendUID ____________________________________________________________


    /**Returns message and user number (in parameter)
     *@return message and user number
     *@exception IOException
     *@see class IntUID*/
    public Message receiveUID(IntUID UID) throws IOException
    {
      int tmp=in.readInt();
      UID.setUID(tmp); 
      return receive();
    }// receiveUID _________________________________________________________


    /**Returns name of the host with which connection is established*/
    public String getHost()
    {
      return host;
    }// getHost ____________________________________________________________
    
    
    public String getIP()
    {
      return IP;
    }// getIP ______________________________________________________________


    /**Closes connection*/
    public void close()
    {
      //System.out.println(CL+".close()");
      try
      {
	out.flush();
	in.close();
	out.close();
	s.close();
      }
      catch(Exception e)
      {
	System.err.println(CL+".close(): exception!: "+e);
	e.printStackTrace();
      }
    }// close ______________________________________________________________


    public String toString()
    {
      return CL+"[socket="+s+"]";
    }// toString ___________________________________________________________


}// Net ////////////////////////////////////////////////////////////////////

