// $Id: RjoinMessage.java,v 1.4 1998/07/10 20:15:37 beca Exp $
/*
File: RjoinMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;


/**
 * A class representing RJOIN message
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class RjoinMessage extends Message
{
    private static final String CL="RjoinMessage";


    private int AT;
    private int SID;
    private int remoteUID;


 /**
   * Creates a new RJOIN message
   *@param ATparam application type
   *@param SIDparam session number
   *@param remoteUIDpar local user number
   **/

  public RjoinMessage(int ATparam, int SIDparam, int remoteUIDpar) {

   AT = ATparam;
   SID = SIDparam;
   remoteUID = remoteUIDpar;

   type = Const.RJOIN;
  }

 /**
   * Creates a new RJOIN message, information is read from the input stream
   *@param instream  input stream
   **/


  RjoinMessage(DataInputStream instream) throws IOException {
    

    AT = instream.readInt();    
    SID = instream.readInt();
    remoteUID = instream.readInt();

    type = Const.RJOIN;
    
  }

 /**
   * Writes RJOIN message to the output stream
   *@param outstream  output stream
   **/

  void send(DataOutputStream outstream) throws IOException {
  

    //System.out.println("Sending type " + Const.RJOIN);
    outstream.writeInt(Const.RJOIN);
    //System.out.println("Sending AT " + AT);
    outstream.writeInt(AT);     
    //System.out.println("Sending SID " + SID);
    outstream.writeInt(SID);
    //System.out.println("Sending remoteUID " + remoteUID);
    outstream.writeInt(remoteUID);

    outstream.flush();

  }

  /**
   * Returns application type
   **/  

  public int getAT() {
       
    return AT;

  } 
 
 /**
   * Returns session number
   **/

  
  public int getSID() {
   
    return SID;
  }

 /**
   * Returns remote user number
   **/

  public int getRemoteUID() {
   
    return remoteUID;
  }
  
    public String toString()
    {
      return CL+"[AT="+AT+",SID="+SID+",rUID="+remoteUID+"]";
    }// toString __________________________________________________________


}// RjoinMessage //////////////////////////////////////////////////////////


