// $Id: TangoServer.java,v 1.5 1998/10/29 19:17:40 beca Exp $
/*
File: TangoServer.java
Component: Server
Date		Author		Changes
Aril 16 98 	Lukasz Beca	Created
11May98         TomaszMajor     metaserver, main params, coherent logging, cleaning
*/


package webwisdom.tango.server;


import java.io.*;
import java.net.*;
import java.util.GregorianCalendar;

import webwisdom.tango.threads.ComThread;
import webwisdom.tango.threads.ThreadIn;
import webwisdom.tango.threads.ThreadOut;
import webwisdom.tango.threads.DbThread;
import webwisdom.tango.threads.Timeout;
import webwisdom.tango.structures.UserRegistry; 
import webwisdom.tango.meta.*;
  

/**Tango server.*/
public final class TangoServer extends Thread implements TangoMetaClient
{
    private static final String CL="TangoServer";


    private static GregorianCalendar expirationDate;

    private int port;
    private ServerSocket socket=null;
    

    static /*sets expirationDate*/
    {
      int yy=1999^0x2864afb4;
      int mm=   6^0x093cba45;
      int dd=   1^0x32838ecf;
      int hh=   0^0x38a47e38;   
      int mn=   0^0x7e39c39a;
      int ss=   0^0xe3987a38;
      expirationDate=new GregorianCalendar(yy^0x2864afb4,
					   mm^0x093cba45,
					   dd^0x32838ecf,
					   hh^0x38a47e38,
					   mn^0x7e39c39a,
					   ss^0xe3987a38);
    }// static _____________________________________________________________


    /**
     *@return license expiration date*/
    public static GregorianCalendar expirationDate()
    {
      return expirationDate;
    }// expirationDate _____________________________________________________


    /**
     *@param p port on which the server is to listen for clients
     *@exception IOException if cannot start service at the given port*/
    public TangoServer(int p) throws IOException
    {
      socket=new ServerSocket(p);
      port=socket.getLocalPort();
    }// TangoServer ___________________________________________________________


    /**Starts server.*/
    public void run()
    {
      try
      {
	ComThread comThread=new ComThread();
	DbThread dbThread=new DbThread();
	Timeout timeout=new Timeout(5*60*1000,comThread);
	UserRegistry userReg = new UserRegistry();
	comThread.start();   
	dbThread.start();
        
	System.out.println("listening on port: "+port);
   	while(true)
	{
	  Socket s=socket.accept();
	  if(expirationDate.before(new GregorianCalendar()))
	  {
	    System.out.println("evaluation period expired!: terminating");
	    return;
	  }
	  System.out.println("accepted connection from: "+socketToString(s));
	  ThreadOut threadOut=new ThreadOut(s);   
	  ThreadIn i=new ThreadIn(comThread,dbThread,threadOut,s,timeout, userReg);
	 
	}
	
      }
      catch(Exception e)
      {
	System.err.println("server error: "+e);
	e.printStackTrace();
      }
    }// run ________________________________________________________________
    
    
    /**Registers this server for meta services.
     *@param h,p metaserver location
     *@exception IOException if cannot connect to the given meta server*/
    public void metaRegister(String h,int p) throws IOException
    {
      new TangoMetaAgent(this,h,p);
    }// metaRegister _______________________________________________________


    static String socketToString(Socket s)
    {
      return "["+s.getInetAddress().getHostName()+":"+s.getPort()+"]";
    }// toString ___________________________________________________________


    public String toString()
    {
      return "TangoServer["+port+"]";
    }// toString ___________________________________________________________


    /**Runs the server at the given port.
       Logs monitoring messages in the specified file.
       Should be called with 2 parameters: port number, and log file.*/
    public static void main(String args[])
    {
      if(expirationDate.before(new GregorianCalendar()))
      {
	System.err.println(CL+": evaluation period expired!: terminating"); 
	System.err.println(CL+": (to renew the license contact marek@npac.syr.edu)"); 
	System.exit(3);
      } 
      
      if(args.length!=2)
      {
	System.err.println(CL+": incorrect number of parameters!");
	System.err.println(CL+": Usage: main(<port#>,<logFile>)");
	System.exit(2);
      }
      
      int port=-1;
      try
      {
	port=Integer.parseInt(args[0]);
      }
      catch(NumberFormatException e)
      {
	System.err.println(CL+": malformed <port#>!");
	System.err.println(CL+": Usage: main(<port#>,<logFile>)");
	System.exit(2);
      }

      PrintStream logStream=null;
      try
      {
	FileOutputStream f=new FileOutputStream(args[1]);//"../logs/server_log"
	logStream=new PrintStream(f);
      }
      catch(IOException e)
      {
	System.err.println(CL+": file \""+args[1]+"\" cannot be open!");
	System.err.println(CL+": Usage: main(<port#>,<logFile>)");
	System.exit(2);
      }
      
      TangoServer server=null;
      try
      {
	server=new TangoServer(port);
      }
      catch(IOException e)
      {
	System.err.println(CL+": cannot start server on port "+port+"!");
	System.exit(1);
      }	

      System.setOut(logStream);
      System.setErr(logStream);

      System.out.println("Tango Interactive Server v.1.01");
      System.out.println("copyright: NPAC, Syracuse University");
      System.out.println("report problems to: marek@npac.syr.edu");
      System.out.println("free evaluation will end on: "+
			 expirationDate.getTime());

      server.start();

      String metaServerHost="beast";
      int metaServerPort=32831;
      try
      {
	server.metaRegister(metaServerHost,metaServerPort);
      }
      catch(IOException e)
      {
	System.err.println(CL+": cannot register at: "+
			   metaServerHost+":"+metaServerPort+"!");
      }	
    }// main _______________________________________________________________

    
}// TangoServer ////////////////////////////////////////////////////////////


