// $Id: AppletHandle.java,v 1.7 1998/11/03 00:53:05 piotr Exp $
/*
File: AppletHandle.java
Component: Plugin
Date		Author		Changes		
Sept 09 96	Lukasz Beca	Created
07May98         TomaszMajor     caTermination() modified
*/

package webwisdom.tango.structures;

import webwisdom.tango.structures.Handle;
import webwisdom.tango.interfaces.AppletBaseInterf;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.EmptyMessage;
import webwisdom.tango.messages.Message;
import webwisdom.tango.messages.EventMessage;
import webwisdom.tango.consts.Const;
//import netscape.javascript.JSObject;
import webwisdom.tango.structures.LocalApplications;


public abstract class AppletHandle implements Handle, Runnable
{
    private static final String CL="AppletHandle";


    protected AppletBaseInterf app;
    private int AID;
    private LocalApplications lapps;
    private MessageQueue q;
    private boolean exit; 
    private Thread th;
    private int closeMode;

 
    public AppletHandle(AppletBaseInterf a,
			int AIDpar,LocalApplications localpar)
    {
      app = a;
      AID = AIDpar;
      lapps = localpar;
      q = new MessageQueue();
      th = new Thread(this);
      exit = false;
      closeMode = Const.NONE;
      th.start();
    }// AppletHandle _______________________________________________________


    //interface for LocalApplications
    //public void sendLAToApp(Message m);
    //public synchronized void finish();
    //public synchronized void setFocus();


    public void sendLAToApp(Message m)
    {
      q.putMessage(m);
    }// sendLAToApp ________________________________________________________
  

    public synchronized void finish()
    {
      if(closeMode==Const.NONE)
      {
	//System.out.println("AppletHandle: finishing application");
	closeMode = Const.CACLOSED;
	caTermination();
      }
    }// finish _____________________________________________________________


    public synchronized void setFocus()
    {
      if(Const.inNetscape())
	{
	  netscape.javascript.JSObject win=(netscape.javascript.JSObject) app.getJSWindow();
	  netscape.javascript.JSObject parent=(netscape.javascript.JSObject)win.getMember("top");
	  if(parent==null)
	    {
	      System.out.println(CL+"setFocus(): unable to get reference to \"top\"!");
	      return;
	    }
	  parent.eval("focus();");
	}
      else
	{
	  java.awt.Frame f=(java.awt.Frame)app.getJSWindow();
	  f.requestFocus();
	  f.toFront();
	}
    }// setFocus ___________________________________________________________
  

    //interface for AppletBase
    //public void sendAppToOther(AppEventMessage m);
    //public void sendAppToOther(AppEventMessage m, int userlist[]);
    //public synchronized void remove();


    public void sendAppToOther(AppEventMessage m)
    {
      lapps.sendAppToOther(AID, m);
    }// sendAppToOther _____________________________________________________
  

    public void sendAppToOther(int userlist[],AppEventMessage m)
    {
      lapps.sendAppToOther(AID,userlist,m);
    }// sendAppToOther _____________________________________________________


    public synchronized void remove()
    {
      if(closeMode==Const.NONE)
      {
	closeMode = Const.WINDOWCLOSED;
	//System.out.println(CL+"remove(): AID="+AID);
	lapps.terminated(AID);
	exit = true;
	q.unblock();    
      }
    }// remove _____________________________________________________________


    //interface for Thread
    //public void run();


    public void run()
    {
      //System.out.println(CL+".run(): AID="+AID);
      while(!exit)
      {
	//System.out.println("ThreadOut: Thread number " + thisno + " sending");
	Message m=q.getMessage();
	synchronized(this)
	{
	  if(!exit)
	  {
	    AppEventMessage am=(AppEventMessage)m;
	    app.receive(am);
	  }
	}    
      }
      //System.out.println("AppletHandle: message queue released");      
      shutdown();
    }// run ________________________________________________________________


    //internal methods
    //private void caTermination();
    //private void shutdown();

    protected abstract void caTermination();
 
    //private void windowTermination(){}

    private void shutdown()
    {
      q.clear();
      q = null;
      app = null;
      lapps = null;
      th = null;
    }// shutdown ___________________________________________________________


}// AppletHandle ///////////////////////////////////////////////////////////


