// $Id: CentralServer.java,v 1.4 1998/11/03 00:53:05 piotr Exp $
/*
File: CentralServer.java
Component: Plugin
Date		Author		Changes
Mar 07 97	Lukasz Beca	Created
*/

package webwisdom.tango.structures;

import webwisdom.tango.structures.ControlApplication;
import webwisdom.tango.structures.LocalApplications;
import webwisdom.tango.threads.DemonThreadIn;
import webwisdom.tango.threads.ThreadOut;
import webwisdom.tango.messages.*;
import java.net.Socket;
import webwisdom.tango.consts.Const;
//import netscape.security.PrivilegeManager;


public class CentralServer
{
    private static final String CL="CentralServer";


    private ControlApplication ca;
    private LocalApplications apps;
    private DemonThreadIn in;
    private ThreadOut out;
    private String serverhost;
    private int serverport;


    public CentralServer(String serverpar,int portpar)
    {
      ca = null;
      apps = null;
      in = null;
      out = null;
      serverhost = serverpar;
      serverport = portpar;
    }


    //interface for LocalBase
    //public void initAssociations(ControlApplication capar,LocalApplications lapar);
 

    public void initAssociations(ControlApplication capar,
				 LocalApplications lapar)
    {
      ca = capar;
      apps = lapar;
      initSocket();
    }
   

    //interface for ControlApplication
    //public void receiveFromCA(Message m);
    //public void logout();
    //public String getServerHost();


    public void sendCAToCS(Message m)
    {
      out.putMessage(m);
    }
  

    public String getServerHost()
    {
      return serverhost;
    }


    public void logout()
    {
      in.logout();
    }
  

    //interface for LocalApplications
    //public void receiveFromApp(Message m);
    //public void started(int AID);


    public void sendLAToCS(Message m)
    {
      out.putMessage(m);
    }


    public void started(int AID, int SID)
    {
      in.started(AID, SID);
    }
       

    //interface for DemonProtocol
    //public void sendCSToCAB(Message m);
    //public void sendCSToApp(Message m);


    public void sendCSToCAB(Message m)
    {
      ca.sendCSToCAB(m);                
    }


    public void sendCSToApp(EventMessage m)
    {
      apps.sendCSToApp(m);
    }


    //interface for DemonThreadIn
    //public void connectionBrk();


    public void connectionBrk()
    {
      ca.errorServer(Const.CONNBROKEN);
      end();
    }


    //internal methods
    //private void initSocket();
    //public void end();


    public void initSocket()
    {
      Socket s;
      if (serverhost.compareTo("*") == 0)
      {
	serverhost = null;
	serverport = 0;     
	ca.errorServer(Const.FILEWRONGFORMAT);
	return;
      }
      if(serverport <= 0)
      {
	serverhost = null;
	serverport = 0;      
	ca.errorServer(Const.FILEWRONGFORMAT);
	return;
      }
      if(Const.inNetscape())
	{
	  netscape.security.PrivilegeManager.enablePrivilege("30Capabilities");
	}
      try
      {
	s=new Socket(serverhost,serverport);
      }
      catch(Exception e)
      {
	s = null;
	System.out.println(CL+".initSocket(): unable to contact server: "+e);
	e.printStackTrace();
	ca.errorServer(Const.UNABLETOCONNECT);
	return;
      }
      out = new ThreadOut(s);   
      in = new DemonThreadIn(out, s, this);
    }// initSocket _________________________________________________________
 

    public void end()
    {
      //   System.out.println("CentralServer: releasing references"); 
      ca = null;
      apps = null;
      if (in != null)
      {
	in.stop();
	in = null;
      }
      if (out != null)
      {
	out.stop();   
	out = null;
      }
      serverhost = null;
      serverport = 0;
    } 


}// CentralServer //////////////////////////////////////////////////////////

