// $Id: ControlApplication.java,v 1.10 1998/11/03 22:17:03 beca Exp $
/*
  File: ControlApplication.java
  Component: Plugin
  Date		Author		Changes
  Mar 07 97	Lukasz Beca	Created
  Nov 20 97   K Olszewski changed method of applet starting
*/
 
package webwisdom.tango.structures;

import webwisdom.tango.messages.*;
import webwisdom.tango.structures.LocalBase;
import webwisdom.tango.structures.LocalApplications;
import webwisdom.tango.structures.CentralServer;
import webwisdom.tango.interfaces.ControlAppletBaseInterf;
import netscape.javascript.JSObject;
import webwisdom.tango.structures.Registrar;
import webwisdom.tango.structures.MessageQueue;
import webwisdom.tango.interfaces.PureJavaInterface;
import webwisdom.tango.consts.Const;
import java.util.Hashtable;

import java.lang.*;
import java.net.*;
import java.io.*;


public class ControlApplication implements Runnable
{
  private static final String CL="ControlApplication";


  private LocalBase lb;
  private CentralServer cs;
  private ControlAppletBaseInterf cab;
  private LocalApplications apps;
  private Registrar reg;
  private int wincounter;
  private Thread th;
  private MessageQueue q;
  private boolean noexit;
  private int endMode;
  private int init_status;
  private Hashtable frames;  // stores references to the frames 

    
  //  private JSObject targetFrame; // prototype
    
  public ControlApplication()
    {
      wincounter = 1;
      lb = null;
      cs = null;
      cab = null;
      apps = null;
      reg = null;
      th = null;   
      noexit = true;
      q = new MessageQueue();
      endMode = Const.NONE;
      init_status = Const.OK;
      
      
      //      targetFrame = null; // frame reference initialization
  
      frames = new Hashtable();        
    }

  //interface for LocalApplications
  //public void receiveFromApplication(int AID, Message m);
  //public void removeApplication(int AID);
  //public void errorApplication(int AID);

    
  public void sendLAToCAB(int AID, AppEventMessage am)
    {
      EventMessage em = new EventMessage(AID, am.getData());
      q.putMessage(em);
    } 
  

  public void removeApplication(int AID)
    {
      cab.removeApplication(AID);
    }
    

  public void errorApplication(int AID)
    {
    }


  //interface for CentralServer
  //public void receiveFromServer(Message m);
  //public void errorServer(int errorCode);
    
    
  public void sendCSToCAB(Message m)
    {
      q.putMessage(m);
    }


  public void errorServer(int errorCode)
    {
      System.out.println(CL+".errorServer("+errorCode+")");
      if(endMode == Const.NONE)
	{
	  switch(errorCode)
	    {
	    case Const.CONNBROKEN: 
	      System.out.println(CL+".errorServer("+errorCode+"): broken connection!");
	      endMode = Const.CONNECTIONEND; 
	      cab.error(Const.CONNBROKEN); 
	      apps.finishAll();
	      apps.end();
	      reg.end();
	      noexit = false;
	      q.unblock(); 
	      break;
	    case Const.UNABLETOCONNECT:
	      System.out.println(CL+".errorServer("+errorCode+"): unable to connect");
	      endMode = Const.CONNECTIONEND;
	      init_status = Const.UNABLETOCONNECT;	 
	      break;	  
	    }
	}
      else
	{
	  System.out.println(CL+".errorServer("+errorCode+"): closed connection");
	}   
    }


  //interface for LocalBase
  //public void errorLocalBase(int errCode);
  //public void initAssociations(LocalBase lbpar,CentralServer cspar,LocalApplicatons lapar,Registrar regpar);
  //public void terminated();


  public void initAssociations(LocalBase lbpar,
			       CentralServer cspar,
			       LocalApplications lapar,
			       Registrar regpar)
    {
      lb = lbpar;
      cs = cspar;
      apps = lapar;
      reg = regpar;
    }
  
    
  public void errorLocalBase(int errCode)
    {
      System.out.println(CL+".errorLocalBase("+errCode+")");
    }


  public void terminated()
    {
      System.out.println(CL+".TERMINATED");
      if (endMode == Const.NONE)
	{
	  endMode = Const.TERMINATEDEND;
	  emergLogout();
	  apps.finishAll();
	  apps.end();
	  reg.end();
	  noexit = false;
	  q.unblock();
	}
      if(!Const.inNetscape())
	{
	  PureJavaInterface.getPureJavaInterface().quit();
	}
    }


  //interface for ControlAppletBaseInterf
  //public void sendCABToCS(Message m);
  //public void sendCABToApp(int AID, Message m);
  //public void finish(int AID);
  //public void logout();
  //public boolean appExists(int AID);
  //public void startApplet(String url, int AID, String height, String width);
  //public void startApplet(String url, int AID, String framename);
  //public void startApplication(int AT, int AID);
  //public void startApplication(String path, String name, String args);
  //public void setFocus(int AID);
  //public int getStatus();
  //public String getServerHost();


  public void sendCABToCS(Message m)
    {
      cs.sendCAToCS(m);
    /*  System.out.println(CL+".send() type:" + m.getType());
      if (m.getType() == Const.GROUP_EVENT) {
      
	      GroupEventMessage gem = (GroupEventMessage) m;
	      System.out.println(CL+".send() group event:" + gem.getData());	      
      
      } */

    }

  
  public void sendCABToApp(int AID, Message m)
    {
      AppEventMessage am = (AppEventMessage) m;
      apps.sendCAToApp(AID, am);
    }
  

  public void finish(int AID)
    {
      apps.finish(AID);
    }


  public void logout()
    {
      try
	{
	  if (endMode == Const.NONE)
	    {
	      System.out.println(CL+".logout()");
	      endMode = Const.LOGOUTEND;
	      apps.finishAll();
	      apps.end();
	      reg.end();
	
	      noexit = false;
	      q.unblock(); 
	    }
	}
      catch(Exception e)
	{
	}
      if(!Const.inNetscape())
	{
	  PureJavaInterface.getPureJavaInterface().quit();
	}
    }
    

  public boolean appExists(int AID)
    {
      return apps.appInSession(AID);
    }
  
  
  public String getServerHost()
    {
      return cs.getServerHost();
    }
  

  public synchronized void startApplet(String url,int AID,int SID,int AT,
				       String height,String width)
    {
      String winname=new String("win"+AID+"_"+SID+"_"+AT);
      String options=new String("\"toolbar=no"+
				",location=no"+
				",directories=no"+
				",status=no"+
				",menubar=no"+
				",resizable=no"+
				",scrollbars=no"+
				",width="+width+
				",height="+height+"\"");
      String com=new String("open(\""+url+"\",\""+winname+"\","+options+");");

      if(Const.inNetscape())
	{
	
	  netscape.javascript.JSObject caframe=(netscape.javascript.JSObject)cab.getJSWindow();
	  if(caframe==null)
	    {
	      System.err.println(CL+".startApplet(): unable to get reference to frame");    
	      return;
	    } 
      
	  netscape.javascript.JSObject win=(netscape.javascript.JSObject)caframe.getMember("top");
	  if(win==null)
	    {
	      System.out.println(CL+".startApplet(): unable to get reference to top window");       
	      return;
	    } 
      
	  //netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserAccess");
      
	  netscape.javascript.JSObject newwin=(netscape.javascript.JSObject)win.eval(com);    
	  //     System.out.println(CL+".startApplet(): evaluating \""+com+"\"");
	}
      else
	{
	 // String table[]={url.toString(),winname};
	 // PureJavaInterface.getPureJavaInterface().startApplet(table);
	 
	 startScript(url, AID, SID, AT);
	}
      
           
    }// startApplet _______________________________________________________


  public synchronized void startAppletInFrame(String url, int AID,int SID,int AT, String frameName)
    // frameName parameter must be added   
    
    {
      if(Const.inNetscape())
	{
	  netscape.javascript.JSObject caFrame;
	  netscape.javascript.JSObject mainWindow;
	  netscape.javascript.JSObject targetFrame, targetWindow;
	  //      String frameName = "chatframe"; only for tests
	  String newFrameName=new String("win"+AID+"_"+SID+"_"+AT);
	  JSObject tmp;
	  String tmpName;
	  Object args[] = new Object[3]; 
      
	  System.out.println(CL + ": frame " + frameName);
	  System.out.println(CL + ": temporary frame " + newFrameName);

      
	  if (!frames.containsKey(frameName))
	    {
      
	      System.out.println(CL + ": frame added");
	      caFrame=(netscape.javascript.JSObject)cab.getJSWindow();
	      mainWindow = (netscape.javascript.JSObject)caFrame.getMember("top"); 	          
	      targetFrame = (netscape.javascript.JSObject) mainWindow.getMember(frameName);
	      targetWindow = (netscape.javascript.JSObject) targetFrame.getMember("window");
	      frames.put(frameName, targetWindow);
	      targetWindow.setMember("name", newFrameName);	
	      
	    }
	  else
	    {
       
	      System.out.println(CL + ": frame retrieved");
	      targetWindow = (netscape.javascript.JSObject) frames.get(frameName);
	
	    }
      
	  targetWindow.setMember("name", newFrameName);	
      
	  tmpName = (String) targetWindow.getMember("name");
	  System.out.println(CL + ": frame name retrieved once again " + tmpName);

        
	  args[0] = url;
	  args[1] = newFrameName; 
	  args[2] = null; 

	  System.out.println("ARGS: " + (String) args[0] + " " + (String) args[1]); 

	  tmp = (netscape.javascript.JSObject) targetWindow.call("open", args);
 
	  System.out.println(CL + ": applet running");
	}
      else
	{
	  startApplet(url, AID, SID, AT,"","");
	}
    }// startAppletInFrame _________________________________________________


  public void startApplication(String path,int AID,int SID,int AT)
    {
      if(Const.inNetscape())
	{
	  lb.startApplication(path,AID,SID,AT,reg.getLocalPort());
	}
      else
	{
	  PureJavaInterface.getPureJavaInterface().startApplication(path,AID,SID,AT);
	}
    }// startApplication ___________________________________________________  

  public void startScript(String path,int AID,int SID,int AT)  {
  
      if(Const.inNetscape()) {
//     	  lb.startApplication(path,AID,SID,AT,reg.getLocalPort());
     	} else {			      
	
	      PureJavaInterface.getPureJavaInterface().startScript(path,AID,SID,AT);
	    }
	    
  }// startApplication ___________________________________________________  

    


  public void setFocus(int AID)
    {
      apps.setFocus(AID);
    }


  public int getStatus()
    {
      return init_status;
    }


  //interface for Registrar
  //public void init(ControlAppletBaseInterf cabparam);
  //public void errorRegistrar(int errorCode);
  //public void addApplication(int AID, int SID, int AT);


  public void init(ControlAppletBaseInterf cabparam)
    {
      cab = cabparam;
      th = new Thread(this);
      th.start();
    }


  public void errorRegistrar(int errorCode)
    {
    }
  
  
  public void addApplication(int AID,int SID,int AT)
    {
      cab.addApplication(AID,SID,AT);
    }


  public String getDefaultContent(String frameName) { // must be changed
    
    // presented frameName is modified, original frame name must be
    // retrieved
    
    //    System.out.println(CL + ": default content retrieved");
    //    return "http://trurl.npac.syr.edu/tango102/ca/null1.html";
  
    return cab.getDefaultContent(frameName);
          
  }

  //internal methods
  //private void run();
  //private void endAll();
  //private void emergLogout();


  public void run()
    {
      while(noexit)
	{
	  //System.out.println(CL+": Thread number "+thisno+" sending");
	  Message m=q.getMessage();
	  if(m.getType()!=Const.EVENT)
	    {
	      cab.receiveServer(m);
	    }
	  else
	    {
	      EventMessage em=(EventMessage)m;
	      int AID=em.getAID();
	      if(AID==Const.CRLAID)
		{
		  cab.receiveServer(m);
		}
	      else
		{
		  AppEventMessage am=new AppEventMessage(Const.CONTROL,em.getData());
		  cab.receiveApp(AID,am);
		} 
	    }
	}
      endAll();
    }// run ________________________________________________________________


  private void endAll()
    {
      //   System.out.println("ControlDemon: releasing references");   
      cs = null;
      apps = null;
      reg = null;
      lb = null;
      cab = null;
      q.clear();
      frames.clear();
      q = null;    
      th = null;
    }// endAll _____________________________________________________________


  private void emergLogout()
    {
      cs.logout();
    }// emergLogout ________________________________________________________


  public String toString()
    {
      return CL;
    }// toString ___________________________________________________________


}// ControlApplication /////////////////////////////////////////////////////



