// $Id: ThreadIn.java,v 1.5 1998/10/29 19:17:57 beca Exp $
/*
File: ThreadIn.java
Component: Server
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/


package webwisdom.tango.threads;


import java.lang.*;
import java.net.*;
import java.io.*;
import webwisdom.tango.messages.Net;
import webwisdom.tango.messages.*;
import webwisdom.tango.messages.LogoutMessage;
import webwisdom.tango.threads.ThreadOut;
import webwisdom.tango.consts.Const;
import webwisdom.tango.protocols.ServerProtocol;
import webwisdom.tango.test.Info;
import webwisdom.tango.structures.UserRegistry;

/**Server thread.*/
public class ThreadIn extends ServerProtocol implements Runnable
{
    private static final String CL="ThreadIn";
  

    private boolean exit;
    private Thread th;   //reference to this thread 
    private UserRegistry userReg;

 
    public ThreadIn(ComThread cparam,DbThread dparam,
		    ThreadOut outparam, Socket s, Timeout t, UserRegistry uReg)
    {
      super(outparam, cparam, dparam, t);   
      n = new Net(s);    
      th = new Thread(this);
      userReg = uReg;
      exit = false;
      th.start();
      userReg.connect(this);
    }// ThreadIn ___________________________________________________________

  
    public void run()
    {
      System.out.println(CL+".run()");
      while(!exit)
      {
	Message m;
	try
	{
	  m=n.receive();
	}
	catch(IOException e)
	{
	  System.err.println(CL+": broken connection!: "+e);
	  e.printStackTrace();
	  if(UID!=0)
	  {
	    logout();
	  }
	  m=null;
	  stop();	 
	  break;
	}
	if(m!=null)
	{
	  int type=m.getType();
	  if(type==Const.EVENT)
	  {
	    handleEVENT((EventMessage)m);
	  }
	  else
	  {
	    switch(type)
	    {
	      case Const.LOGIN:
		handleLOGIN((LoginMessage)m);
		if (UID != 0) 
		   userReg.addUser(UID, this);
		break;
	      case Const.LOGOUT:
		handleLOGOUT((LogoutMessage)m);
		stop(); //tests
		break;
	      case Const.JOIN:
		handleJOIN((JoinMessage)m);
		break;
	      case Const.JANS:
		handleJANS((JansMessage)m);
		break;
	      case Const.FINISH:
		handleFINISH((FinishMessage)m);
		break;
	      case Const.MASTER:
		handleMASTER((MasterMessage)m);
		break;
	      case Const.MANS:
		handleMANS((MansMessage)m);
		break;
	      case Const.RJOIN:
		handleRJOIN((RjoinMessage)m);
		break;
	      case Const.RANS:
		handleRANS((RansMessage)m);
		break;  
	      case Const.STARTED:
		handleSTARTED((StartedMessage)m);
		break;
	      case Const.PING:
	  	handlePING((PingMessage)m);
		break;
	      case Const.GROUP_EVENT:
		handleGROUP_EVENT((GroupEventMessage)m);
		break;
	      default:
		System.err.println(CL+": unrecognized message type!: "+type);
	    }
	  }
	}
	else
	{
	  System.err.println(CL+": null message!");
	}
      }  
    }// run ________________________________________________________________


    public void stop()
    {
      //System.out.println(CL+".stop()");
      exit=true;
      userReg.removeUser(UID, this); 
      out.stop();
      n.close();
    }// stop _______________________________________________________________


}// ThreadIn ///////////////////////////////////////////////////////////////

